/**
 * DimpCore.js - Dimp UI application logic.
 *
 * $Horde: dimp/js/src/DimpCore.js,v 1.369.2.10 2008/01/18 20:19:12 slusarz Exp $
 *
 * Copyright 2005-2008 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

var baseWindow = false,
    DMenu,
    /* Trick some Horde js into thinking this is the parent Horde window. */
    frames = { horde_main: true };

/* DimpCore object. */
var DimpCore = {
    // Vars used and defaulting to null/false:
    //   viewport, folder, uid, ppID, lastrow, folderswitch, filtertoggle,
    //   inAjaxCallback, pollPE, isvisible, offset, is_logout, fl_visible,
    //   searchobserve
    compose_id: 1,
    drags: $H(),
    remove_gc: [],
    selected: [],
    server_error: 0,
    showPreview: DimpVar.preview_pref,

    buttons: [ 'button_reply', 'button_forward', 'button_spam', 'button_ham', 'button_delete' ],

    sfiltersfolder: $H({ sf_all: 'all', sf_current: 'current' }),
    sfilters: $H({ sf_msgall: 'all', sf_from: 'from', sf_to: 'to', sf_subject: 'subject' }),

    debug: function(label, e)
    {
        if (!this.is_logout && DimpVar.debug) {
            if (Object.isString(e.message)) {
                e = e.message;
            }
            alert(label + ': ' + e);
        }
    },

    // Message selection functions
    select: function(id, opts)
    {
        opts = opts || {};
        var uflag = [];
        if (!opts.add && this.selectedCount()) {
            this.clearSelected();
        }

        id.each(function(s) {
            s = parseInt(s);
            if (opts.add) {
                if (!this.isSelected(s)) {
                    this.selected.push(s);
                }
            } else {
                this.selected = [ s ];
            }

            if (this.viewport.inBuffer(s)) {
                uflag.push(s);
                if (this.lastrow != -1) {
                    this.lastrow = parseInt(this.viewport.getRowById(s).rownum);
                }
            }
        }.bind(this));

        if (uflag.size()) {
            this.viewport.updateFlag(uflag, 'selectedRow', true, false);
        }
        this.toggleButtons();

        if (!opts.nopreview && $('previewPane').visible()) {
            if (opts.right) {
                this.clearPreviewPane();
            } else {
                this.initPreviewPane();
            }
        }
    },

    deselect: function(id, opts)
    {
        opts = opts || {};
        this.selected = (opts.clearall) ? [] : this.selected.without.apply(this.selected, id);
        if (id.size()) {
            this.viewport.updateFlag(id, 'selectedRow', false, false);
        }

        var count = this.selectedCount();
        if (!count) {
            this.lastrow = null;
        }

        if (!opts.clearall) {
            this.toggleButtons();
            if (opts.right || !count) {
                this.clearPreviewPane();
            } else if ((count == 1) && $('previewPane').visible()) {
                var row = this.viewport.getRowById(this.selected.first());
                this.loadPreviewPane({ mbox: row.mbox, uid: row.uid });
            }
        }
    },

    ctrlSelect: function(id, e)
    {
        id = parseInt(id);
        var args = { right: e.isRightClick() };
        if (this.isSelected(id)) {
            this.deselect([ id ], args);
        } else {
            this.select([ id ], $H(args).merge({ add: true }).toObject());
        }
    },

    shiftSelect: function(id, e)
    {
        id = parseInt(id);
        var args = { add: true, right: e && e.isRightClick() },
            lastrow = this.lastrow;
        this.select([ id ], args);
        if (this.selectedCount() == 1) {
            return;
        }

        // Logic: link the current id with the last clicked id.
        var row = this.viewport.getRowById(id);
        var bounds = [ row.rownum, lastrow ];
        var start = bounds.min();

        this.processShiftSelect(start, bounds.max() - start + 1);
    },

    processShiftSelect: function(s, l)
    {
        var ids = this.viewport.getRows(s, l);
        if (ids.size() == l) {
            this.shiftSelectAdd(ids.pluck('uid'));
        } else {
            this.msgListLoading(true);
            this.doAction('GetMsgUIDSlice', { start: s, length: l, folder: this.folder }, [], this.shiftSelectCallback.bind(this));
        }
    },

    selectAll: function()
    {
        this.processShiftSelect(1, this.viewport.getMetaData('total_rows'));
    },

    shiftSelectCallback: function(r)
    {
        if (r.response.uids) {
            this.shiftSelectAdd(r.response.uids.parseRangeString());
        }
        this.msgListLoading(false);
    },

    shiftSelectAdd: function(ids)
    {
        var tmp = this.lastrow;
        this.lastrow = -1;
        this.select(ids, { add: true });
        this.lastrow = tmp;
    },

    selectedCount: function()
    {
        return this.selected.size();
    },

    clearSelected: function()
    {
        this.deselect(this.selected, { clearall: true });
    },

    resetSelected: function()
    {
        this.clearSelected();
        this.toggleButtons();
        this.clearPreviewPane();
    },

    isSelected: function(id)
    {
        return (this.selected.indexOf(parseInt(id)) != -1);
    },

    getSelectedRows: function()
    {
        return this.selected.collect(function(s) {
            return { uid: s, mbox: this.folder };
        }.bind(this));
    },

    // absolute = Is num an absolute row number - from 1 -> page_size (true) -
    //            or a relative change from the current selected value (false)
    //            If no current selected value, or more than 1 current
    //            selected value, the first message in the current viewport is
    //            selected.
    moveSelected: function(num, absolute)
    {
        var curr,
            curr_row = (this.selectedCount() == 1) ? this.viewport.getRowById(this.selected.first()) : null;

        if (absolute) {
            if (!this.viewport.getMetaData('total_rows')) {
                return;
            }
            var row = this.viewport.getRows(num, 1);
            curr = num;
        } else {
            if (num == 0) {
                return;
            }

            if (curr_row === null) {
                curr = this.viewport.currentOffset() + 1;
            } else {
                curr = curr_row.rownum + num;
                curr = (num > 1) ? Math.min(curr, this.viewport.getMetaData('total_rows')) : Math.max(curr, 1);
            }
            row = this.viewport.getRows(curr, 1);
        }

        if (row.size()) {
            row = row.first();
            if (!curr_row || row.uid != curr_row.uid) {
                this.viewport.positionById(row.uid);
                this.select([ row.uid ], { nopreview: true });
                // Pause for 300ms before loading preview message.
                (function() { if (this.selectedCount() == 1 && this.selected.first() == row.uid) { this.go('msg:' + this.folder + ':' + row.uid, row); } }).bind(this).delay(0.3);
            }
        } else {
            this.offset = curr;
            this.viewport.requestContentRefresh(curr - 1);
        }
    },
    // End message selection functions

    go: function(loc, data)
    {
        data = data || null;
        this.show(loc, data);
        Horde.dhtmlHistory.add(loc, data);
        return false;
    },

    show: function(loc, data)
    {
        if (loc == 'app:imp') {
            this.go('folder:INBOX');
            return;
        }

        if (loc.startsWith('compose:')) {
            return;
        }

        if (loc.startsWith('msg:')) {
            if (data && this.viewport) {
                this.select([ data.uid ]);
                this.loadPreviewPane(data);
                return;
            }

            // If data is null then we came directly to this page or the user
            // hit Reload. Load the folder view and then go to the specified
            // message.
            var separator = loc.indexOf(':', 4);
            this.uid = loc.substring(separator + 1);
            loc = 'folder:' + loc.substring(4, separator);
            // Now fall through to the 'folder:' check below.
        }

        if (loc.startsWith('folder:')) {
            this.folder = loc.substring(7);
            this.highlightSidebar(this.getFolderId(this.folder));
            $('dimpmain_portal').hide();
            $('dimpmain_folder').show();
            this.loadFolder();
            return;
        }

        $('dimpmain_folder').hide();
        $('dimpmain_portal').update(DimpText.loading).show();

        if (loc.startsWith('app:')) {
            var app = loc.substr(4);
            this.highlightSidebar('app' + app);
            if (data) {
                this.iframeContent(loc, data);
            } else if (DimpVar.app_urls[app]) {
                this.iframeContent(loc, DimpVar.app_urls[app]);
            }
            return;
        }

        switch (loc) {
        case 'portal':
            this.highlightSidebar('appportal');
            this.setTitle(DimpText.portal);
            this.doAction('ShowPortal', {}, [], this.portalCallback.bind(this));
            break;

        case 'options':
            this.highlightSidebar('appoptions');
            this.setTitle(DimpText.prefs);
            this.showOptions();
            break;
        }
    },

    /* 'action' -> if action begins with a '*', the exact string will be used
     *  instead of sending the action to the IMP handler. */
    doAction: function(action, params, uids, callback, opts)
    {
        opts = opts || {};
        params = $H(params);
        action = action.startsWith('*')
            ? action.substring(1)
            : DimpVar.URI_IMP + '/' + action;
        if (uids && uids.size()) {
            params.set('uid', uids.toRangeString());
        }
        if (DimpVar.SESSION_ID) {
            params.update(DimpVar.SESSION_ID.toQueryParams());
        }
        opts.parameters = params.toQueryString();
        if (callback) {
            opts.onComplete = function(t, o) { this.doActionComplete(t, callback); }.bind(this);
        }
        opts.onException = function(r, e) { this.debug('onException', e); }.bind(this);
        opts.onFailure = function(t, o) { this.debug('onFailure', t) }.bind(this);
        new Ajax.Request(action, opts);
    },

    doActionComplete: function(request, callback)
    {
        this.inAjaxCallback = true;
        var error = false, r = {};

        if (!request.responseText || !request.responseText.length) {
            error = true;
        } else {
            try {
                r = request.responseText.evalJSON(true);
            } catch (e) {
                this.debug('doActionComplete', e);
                error = true;
            }
        }

        if (Object.isFunction(callback)) {
            try {
                callback(r);
            } catch (e) {
                this.debug('doActionComplete', e);
            }
        }

        if (error) {
            if (++this.server_error == 3) {
                r = { msgs: [ { type: 'horde.error', message: DimpText.ajax_timeout } ], msgs_auto: true };
            }
        } else {
            if (this.server_error >= 3) {
                r.msgs.push({ type: 'horde.success', message: DimpText.ajax_recover });
            }
            this.server_error = 0;
        }

        if (r.msgs && r.msgs_auto) {
            this.showNotifications(r.msgs);
        }

        // Check if the current folder needs refresh.
        if (this.viewport) {
            this.viewport.checkCacheCallback(r);
        }

        this.inAjaxCallback = false;
    },

    setTitle: function(title)
    {
        document.title = DimpVar.name + ' :: ' + title;
    },

    highlightSidebar: function(id)
    {
        // Folder bar may not be fully loaded yet.
        if ($('foldersLoading').visible()) {
            this.highlightSidebar.bind(this, id).delay(0.1);
            return;
        }

        $('sidebarPanel').select('.on').invoke('removeClassName', 'on');

        var elt = $(id);
        if (!elt) {
            return;
        }
        if (!elt.match('LI')) {
            elt = elt.up();
            if (!elt) {
                return;
            }
        }
        elt.addClassName('on');

        // Make sure all subfolders are expanded
        elt.ancestors().each(function(n) {
            if (n.hasClassName('subfolders')) {
                this.toggleSubFolder(n.id.substring(3), 'exp');
            }
        }.bind(this));
    },

    showNotifications: function(msgs, doc)
    {
        if (!msgs.size() || this.is_logout) {
            return;
        }

        doc = doc || document;

        var alerts = $('alerts');
        if (!alerts) {
            alerts = new Element('DIV', { id: 'alerts' });
            doc.body.appendChild(alerts);
        }

        msgs.find(function(m) {
            switch (m.type) {
            case 'dimp.timeout':
                this.is_logout = true;
                if (parent.frames.horde_main) {
                    parent.location = DimpVar.timeout_url;
                } else {
                    window.location = DimpVar.timeout_url;
                }
                return true;
                break;

            case 'horde.error':
            case 'horde.message':
            case 'horde.success':
            case 'horde.warning':
            case 'imp.reply':
            case 'imp.forward':
            case 'imp.redirect':
            case 'dimp.request':
            case 'dimp.sticky':
                var clickdiv, div;
                if ($w('dimp.request dimp.sticky').indexOf(m.type) != -1) {
                    div = new Element('DIV', { className: m.type.replace('.', '-') }).update(m.message);
                } else {
                    div = new Element('DIV', { className: m.type.replace('.', '-') }).insert(m.message.unescapeHTML().unescapeHTML());
                }
                alerts.appendChild(div);

                // IE6 has a bug that does not allow the body of a div to be
                // clicked to trigger an onclick event for that div (it only
                // seems to be an issue if the div is overlaying an element
                // that itself contains an image).  However, the alert box
                // normally displays over the message list, and we use several
                // graphics in the default message list layout, so we see this
                // buggy behavior 99% of the time.  The workaround is to
                // overlay the div with a like sized div containing a clear
                // gif, which tricks IE into the correct behavior.
                if (DimpVar.is_ie6) {
                    var iefix = new Element('DIV', { className: 'ie6alertsfix' });
                    iefix.clonePosition(div, { setLeft: false, setTop: false });
                    clickdiv = iefix;
                    iefix.appendChild(div.remove());
                    alerts.appendChild(iefix);
                } else {
                    clickdiv = div;
                }

                var fadeeffect = function() { Effect.Fade(div, { duration: 1.5, afterFinish: this.removeAlert.bind(this) }) }.bind(this);

                clickdiv.observe('click', fadeeffect);

                if ($w('horde.error dimp.request dimp.sticky').indexOf(m.type) == -1) {
                    fadeeffect.delay(3);
                }

                if (m.type == 'dimp.request') {
                    var requestfunc = function() { fadeeffect(); document.stopObserving('click', requestfunc) };
                    document.observe('click', requestfunc);
                }
            }
        }.bind(this));
    },

    removeAlert: function(effect)
    {
        try {
            var elt = $(effect.element);
            var parent = elt.up();
            // We may have already removed this element from the DOM tree
            // (if the user clicked on the notification), so check parentNode
            // here - will return null if node is not part of DOM tree.
            if (parent.parentNode) {
                this.remove_gc.push(elt.remove());
                if (!parent.immediateDescendants().size() &&
                    parent.hasClassName('ie6alertsfix')) {
                    this.remove_gc.push(parent.remove());
                }
            }
        } catch (e) {
            this.debug('removeAlert', e);
        }
    },

    checkQuota: function()
    {
        this.doAction('CheckQuota', {}, [], this.checkQuotaCallback.bind(this));
    },

    checkQuotaCallback: function(r)
    {
        if (r.response && r.response.quota) {
            $('quota').update(r.response.quota);
        }
    },

    iframeContent: function(name, location)
    {
        if (name === null) {
            name = location;
        }

        var container = $('dimpmain_portal');
        if (!container) {
            this.showNotifications([ { type: 'horde.error', message: 'Bad portal!' } ]);
            return;
        }

        var iframe = new Element('IFRAME', { id: 'iframe' + 'name', className: 'iframe', frameborder: 0 });
        iframe.writeAttribute('src', location);

        container.insert(iframe);
    },

    msgWindow: function(args)
    {
        this.updateUnseenUID(args.uid, args.folder, 0);
        var url = DimpVar.message_url;
        url += (url.include('?') ? '&' : '?') +
               $H({ folder: args.folder,
                    thismailbox: args.folder,
                    uid: args.uid }).toQueryString();
        this.popupWindow(url, 'msgview');
        return false;
    },

    compose: function(type, args)
    {
        var url = DimpVar.compose_url;
        if (type) {
            var params = $H({ type: type });
            $H(args).keys().each(function(prop) {
                if (prop == 'folder') {
                    params.set('thismailbox', args[prop]);
                }
                params.set(prop, args[prop]);
            });
            url += (url.include('?') ? '&' : '?') + params.toQueryString();
        }
        this.popupWindow(url, '');
    },

    popupWindow: function(url, name)
    {
        if (!window.open(url, name, 'width=' + DimpVar.popup_width + ',height=' + DimpVar.popup_height + ',status=1,scrollbars=yes,resizable=yes')) {
            this.showNotifications([ { type: 'horde.warning', message: DimpText.popup_block } ]);
        }
    },

    closePopup: function()
    {
        // Mozilla bug/feature: it will not close a browser window
        // automatically if there is code remaining to be performed (or, at
        // least, not here) unless the mouse is moved or a keyboard event
        // is triggered after the callback is complete. (As of FF 2.0.0.3 and
        // 1.5.0.11).  So wait for the callback to complete before attempting
        // to close the window.
        if (this.inAjaxCallback) {
            this.closePopup.bind(this).delay(0.025);
        } else {
            window.close();
        }
    },

    logout: function()
    {
        this.is_logout = true;
    },

    composeMailbox: function(type)
    {
        if (!this.selectedCount()) {
            return;
        }
        this.getSelectedRows().each(function(s) {
            this.compose(type, { folder: s.mbox, uid: s.uid });
        }.bind(this));
    },

    loadFolder: function(background)
    {
        this.resetSelected();
        this.searchfilterClose(true);

        $('folderName').update(DimpText.loading);
        $('msgHeader').update();
        this.folderswitch = true;

        if (!this.viewport) {
            this.createViewPort();
        }

        this.viewport.loadView(this.folder, { folder: this.folder, thismailbox: this.folder }, this.uid, background);
    },

    createViewPort: function()
    {
        this.viewport = new ViewPort({
            content_container: 'msgList',
            empty_container: 'msgList_empty',
            error_container: 'msgList_error',
            fetch_action: 'ListMessages',
            cachecheck_action: 'CheckCache',
            msgid_prefix: 'msg',
            template: message_list_template,
            buffer_pages: DimpVar.buffer_pages,
            limit_factor: DimpVar.limit_factor,
            viewport_wait: DimpVar.viewport_wait,
            split_pane: 'previewPane',
            splitbar: 'splitBar',
            content_class: 'msglist',
            row_class: 'msgRow',
            maxHeight: function() {
                return document.viewport.getHeight() - $('msgList').viewportOffset()[1];
            }.bind(this),
            ajaxRequest: function(action, params, callback, opts) {
                this.doAction(action, params, [], callback, opts);
            }.bind(this),
            onSlide: function() {
                this.setMessageListTitle();
            }.bind(this),
            onScrollIdle: function() {
                this.setMessageListTitle();
            }.bind(this),
            onViewChange: function() {
                this.remove_gc = this.remove_gc.concat(this.viewport.visibleRows());
                this.viewport.clear();
            }.bind(this),
            onContent: function(rows) {
                var thread = ((this.viewport.getMetaData('sortby') == DimpVar.sortthread) && this.viewport.getMetaData('thread'));
                rows.each(function(r) {
                    var rid = r.id.substring(3), tn;
                    var row = this.viewport.getRowById(rid);
                    this.viewport.updateFlag([ rid ], 'selectedRow', this.isSelected(rid), false);
                    // Add thread graphics
                    if (thread && thread[row.uid]) {
                        var o, subj = r.down('.msgSubject'), u = thread[row.uid];
                        tn = subj.firstChild;
                        for (var i = 0, length = u.length; i < length; ++i) {
                            o = $('thread_img_' + u.charAt(i)).cloneNode(false);
                            o.id = '';
                            subj.insertBefore(o, tn);
                        }
                    }
                    // Add attachment graphics
                    if (row.atc) {
                        var size = r.down('.msgSize');
                        tn = size.firstChild;
                        var n = $('atc_img_' + row.atc).cloneNode(false);
                        n.id = '';
                        size.insertBefore(n, tn);
                    }
                }.bind(this));
            }.bind(this),
            onComplete: function() {
                if (this.uid) {
                    if (this.viewport.positionById(this.uid)) {
                        this.select([ this.uid ]);
                    }
                    this.uid = null;
                } else if (this.offset) {
                    this.select([ this.viewport.getRows(this.offset, 1).first().uid ]);
                    this.offset = null;
                }

                // 'label' will not be set if there has been an error
                // retrieving data from the server.
                var l = this.viewport.getMetaData('label');
                if (l) {
                    $('folderName').update(l);
                }
                if (this.folderswitch) {
                    this.folderswitch = false;

                    if (this.folder == DimpVar.spam_folder) {
                        if (this.buttons.indexOf('button_spam') != -1) {
                            $('button_spam').up().hide();
                            $('ctx_msg_spam').hide();
                        }
                        if (this.buttons.indexOf('button_ham') != -1) {
                            $('button_ham').up().show();
                            $('ctx_msg_ham').show();
                        }
                    } else {
                        if (this.buttons.indexOf('button_spam') != -1) {
                            $('button_spam').up().show();
                            $('ctx_msg_spam').show();
                        }
                        if (this.buttons.indexOf('button_ham') != -1) {
                            $('button_ham').up().hide();
                            $('ctx_msg_ham').hide();
                        }
                    }

                    $('msgFromLink').update(this.viewport.getMetaData('special') ? DimpText.totext : DimpText.from);
                    this.setSortColumns();
                }

                var threadsort = this.viewport.getMetaData('sortby') == DimpVar.sortthread;
                this.viewport.forceUpdate(threadsort);
                this.setMessageListTitle();

                if (this.filtertoggle) {
                    if (threadsort && this.filtertoggle == 1) {
                        this.setSortColumns(DimpVar.sortdate);
                    } else {
                        this.setFolderLabel(this.folder, this.viewport.getMetaData('unseen'));
                        this.setSortColumns();
                    }
                    this.filtertoggle = 0;
                } else if ($('qoptions').visible()) {
                    this.setFolderLabel(this.folder);
                } else {
                    this.setFolderLabel(this.folder, this.viewport.getMetaData('unseen'));
                }

                this.updateTitle();
                this.msgListLoading(false);
            }.bind(this),
            onFetch: function() {
                this.msgListLoading(true);
            }.bind(this),
            onWait: function() {
                this.showNotifications([ { type: 'horde.warning', message: DimpText.listmsg_wait } ]);
            }.bind(this),
            onFail: function() {
                this.showNotifications([ { type: 'horde.error', message: DimpText.listmsg_timeout } ]);
                this.msgListLoading(false);
            }.bind(this),
            onFirstContent: function() {
                this.clearPreviewPane();
                $('msgList').observe('dblclick', this.handleMsgListDblclick.bindAsEventListener(this));
                $('msgList').observe('mouseover', this.handleMsgListMouseover.bindAsEventListener(this));
            }.bind(this),
            onRemoveRows: function(r) {
                if (r.first()) {
                    this.updateUnseenStatus(r.first().mbox, r.findAll(function(row) {
                        return row.bg.match(new RegExp("(^|\\s)unseen(\\s|$)"));
                    }).size() * -1);
                    this.setMessageListTitle();
                }
            }.bind(this),
            onClearRows: function(r) {
                r.each(function(row) {
                    if (row.id) {
                        this.removeMouseEvents(row.id);
                    }
                }.bind(this));
                this.remove_gc = this.remove_gc.concat(r);
            }.bind(this),
            onBeforeResize: function() {
                this.isvisible = (this.selectedCount() == 1) && (this.viewport.visible(this.selected.first()) == 0);
            }.bind(this),
            onAfterResize: function() {
                if (this.isvisible) {
                    this.viewport.positionById(this.selected.first());
                }
            }.bind(this)
        });

        // DIMP's version of Droppables is smart enough to ignore Draggable
        // items that don't need any droppable listeners.
        $('splitBar').writeAttribute('nodrop', true);

        // Initialize preview pane status.
        this.viewport.showSplitPane(this.showPreview);

        // If starting in no preview mode, need to set the no preview class
        if (!this.showPreview) {
            $('msgList').addClassName('msglistNoPreview');
        }

        // Set up viewport filter events.
        var mf = $('msgList_filter');
        this.viewport.addFilter('ListMessages', this.addSearchfilterParams.bind(this));
        mf.observe('keyup', this.searchfilterStatus.bindAsEventListener(this));
        mf.observe('focus', this.searchfilterOnAction.bind(this, false));
        mf.observe('blur', this.searchfilterOnAction.bind(this, true));

        // Resize message pane on window size change.
        Event.observe(window, 'resize', this.viewport.onResize.bind(this.viewport));

        // Add loading graphic to message list pane.
        var img = $($('loading_img').firstDescendant().cloneNode(false)).hide();
        img.id = 'folderLoading';
        $('msgListLoading').appendChild(img);
    },

    handleMsgListDblclick: function(e)
    {
        var elt = this.getMsgRow(e);
        if (!elt) {
            return;
        }
        var row = this.viewport.getRowById(elt.id.substring(3));
        var hash = { folder: row.mbox, uid: row.uid };
        return row.draft ? this.compose('resume', hash) : this.msgWindow(hash);
    },

    handleMsgListMouseover: function(e)
    {
        e = this.getMsgRow(e);
        if (!e) {
            return;
        }
        this.addMouseEvents(e.id, this.viewport.getRowById(e.id.substring(3)).menutype);
    },

    getMsgRow: function(e)
    {
        e = e.element();
        if (e && !e.hasClassName('msgRow')) {
            e = e.up('.msgRow');
        }
        return e;
    },

    updateTitle: function()
    {
        var label;
        if ($('qoptions').visible()) {
            label = DimpText.search + ' :: ' + this.viewport.getMetaData('total_rows') + ' ' + DimpText.resfound;
        } else {
            var elt = $(this.getFolderId(this.folder));
            if (elt) {
                var unseen = elt.readAttribute('u');
                label = elt.readAttribute('l');
                if (unseen > 0) {
                    label += ' (' + unseen + ')';
                }
            } else {
                label = this.viewport.getMetaData('label');
            }
        }
        this.setTitle(label);
    },

    messageClick: function(r, e)
    {
        var right = e.isRightClick();
        this.select([ r ], { right: right });
        if (!right) {
            var row = this.viewport.getRowById(r);
            return this.go('msg:' + row.mbox + ':' + row.uid, row);
        }
    },

    sort: function(sortby, e)
    {
        // Don't change sort if we are past the sortlimit
        if (this.viewport.getMetaData('sortlimit')) {
            return;
        }

        // As of now, sort() is not always called by a prototype event
        // handler. Thus, we need to manually call extend() on the event ob.
        e = Event.extend(e);

        // See if we're switching between two sorts on the same
        // column.
        var clicked = e.element(), s;
        if (clicked.nodeType == 3) {
            clicked = clicked.parentNode;
        }
        if (clicked.match('small')) {
            // Swap the major and minor sorts on the column.
            sortby = clicked.readAttribute('sortby');
            var pn = clicked.up(), text = clicked.getText();
            clicked.writeAttribute('sortby', pn.readAttribute('sortby'));
            pn.writeAttribute('sortby', sortby);
            clicked.setText(pn.getText());
            pn.setText(text);
        }
        if (sortby == this.viewport.getMetaData('sortby')) {
            s = { sortdir: (this.viewport.getMetaData('sortdir') ? 0 : 1) };
            this.viewport.setMetaData({ sortdir: s.sortdir });
        } else {
            s = { sortby: sortby };
            this.viewport.setMetaData({ sortby: s.sortby });
            if (sortby == DimpVar.sortthread) {
                this.viewport.forceUpdate(true);
            }
        }
        this.setSortColumns();
        this.viewport.reload(s);
    },

    setSortColumns: function(sortby)
    {
        if (Object.isUndefined(sortby)) {
            sortby = this.viewport.getMetaData('sortby');
        }
        var arrsort = (sortby == DimpVar.sortarrival),
            dsort = $('sortdate'),
            ssort = $('sortsubject'),
            tsort = (sortby == DimpVar.sortthread);

        // Set Subject/Thread sort
        ssort.update(tsort ? DimpText.thread : DimpText.subject);
        ssort.writeAttribute('sortby', tsort ? DimpVar.sortthread : DimpVar.sortsubject);

        // Set Date/Arrival sort
        dsort.update(arrsort ? DimpText.arrival : DimpText.datesort);
        dsort.writeAttribute('sortby', arrsort ? DimpVar.sortarrival : DimpVar.sortdate);

        if (!$('qoptions').visible()) {
            ssort.appendChild(new Element('SMALL', { id: 'sortsubject_small', sortby: (tsort ? DimpVar.sortsubject : DimpVar.sortthread) }).insert(tsort ? DimpText.subject : DimpText.thread));
            if (this.viewport.getMetaData('sortlimit')) {
                $('sortsubject_small').hide();
            } else {
                $('sortsubject_small').visible();
            }
        }

        dsort.appendChild(new Element('SMALL', { sortby: (arrsort ? DimpVar.sortdate : DimpVar.sortarrival) }).insert(arrsort ? DimpText.datesort : DimpText.arrival));

        $('dimpmain_folder').select('.item').invoke('immediateDescendants').flatten().each(function(node) {
            if (!node.match('div')) {
                return;
            }
            var setsort = false;
            if (node.hasClassName('msgFrom')) {
                setsort = (sortby == DimpVar.sortfrom);
            } else if (node.hasClassName('msgSubject')) {
                setsort = (sortby == DimpVar.sortsubject || sortby == DimpVar.sortthread);
            } else if (node.hasClassName('msgDate')) {
                setsort = (sortby == DimpVar.sortdate || sortby == DimpVar.sortarrival);
            } else if (node.hasClassName('msgSize')) {
                setsort = (sortby == DimpVar.sortsize);
            }

            if (setsort) {
                this.isSort(node);
            } else {
                this.notSort(node);
            }
        }.bind(this));
    },

    toggleButtons: function()
    {
        var disable = (this.selectedCount() == 0);
        this.buttons.each(function(b) {
            var b_up = $(b).up();
            if (disable) {
                b_up.addClassName('disabled');
            } else {
                b_up.removeClassName('disabled');
            }
            DMenu.disable(b + '_img', true, disable);
        });
    },

    isSort: function(node)
    {
        var sortdir = this.viewport.getMetaData('sortdir');
        node.removeClassName(sortdir ? 'sortdown' : 'sortup');
        node.addClassName(sortdir ? 'sortup' : 'sortdown');
    },

    notSort: function(node)
    {
        node.removeClassName('sortup');
        node.removeClassName('sortdown');
    },

    loadPreviewPane: function(data)
    {
        var pp = $('previewPane');
        if (!pp.visible()) {
            return;
        }
        if (this.ppID &&
            this.ppID.uid == data.uid &&
            this.ppID.mbox == data.mbox) {
            return;
        }
        this.ppID = data;
        this.updateUnseenUID(data.uid, data.mbox, 0);

        // Clear tooltip events.
        this.remove_gc = this.remove_gc.concat($('previewPane').select('a'));

        var img = $('loading_img').firstDescendant().cloneNode(false);
        img.id = 'msgLoading';
        $A(pp.childNodes).each(function(n) {
            pp.removeChild(n);
        });
        pp.appendChild(img);

        this.doAction('ShowPreview', { uid: data.uid, folder: data.mbox }, [], this.loadPreviewPaneCallback.bind(this));
    },

    loadPreviewPaneCallback: function(r)
    {
        if (!r.response || this.selectedCount() != 1) {
            this.clearPreviewPane();
            return;
        }
        r = r.response;
        if (r.index == this.selected.first() && r.folder == this.folder) {
            var pp = $('previewPane');
            pp.update(r.msg);
            this.buildAddressLinks(pp);
            ToolTips.attachBehavior();
        }
    },

    initPreviewPane: function()
    {
        if (this.selectedCount() != 1) {
            this.clearPreviewPane();
        } else {
            this.loadPreviewPane({ mbox: this.folder, uid: this.selected.first() });
        }
    },

    clearPreviewPane: function()
    {
        var p = $('previewPane');
        if (p.visible()) {
            // Clear tooltip events.
            this.remove_gc = this.remove_gc.concat($('previewPane').select('a'));

            var d = $($('previewInfoText').cloneNode(true)).show();
            d.id = 'previewInfo';
            $A(p.childNodes).each(function(n) {
                p.removeChild(n);
            });
            p.appendChild(d);
            this.ppID = null;
        }
    },

    updateUnseenUID: function(uid, mbox, setflag)
    {
        var r = this.viewport.getRowById(uid);
        if (!r) {
            return false;
        }

        var unseen = 0,
            unseenset = r.bg.match(new RegExp("(^|\\s)unseen(\\s|$)"));
        if ((setflag && unseenset) || (!setflag && !unseenset)) {
            return false;
        }

        if (setflag) {
            this.viewport.updateFlag([ r.uid ], 'unseen', true, true);
            ++unseen;
        } else {
            this.viewport.updateFlag([ r.uid ], 'unseen', false, true);
            --unseen;
        }

        return this.updateUnseenStatus(mbox, unseen);
    },

    updateUnseenStatus: function(mbox, change)
    {
        if (change == 0) {
            return false;
        }
        var unseen = parseInt($(this.getFolderId(mbox)).readAttribute('u')) + change;
        this.viewport.setMetaData({ unseen: unseen });
        this.setFolderLabel(mbox, unseen);
        this.updateTitle();
        return true;
    },

    setMessageListTitle: function()
    {
        var rows = this.viewport.getMetaData('total_rows');
        if (rows > 0) {
            var offset = this.viewport.currentOffset();
            $('msgHeader').update(DimpText.messages + ' ' + (offset + 1) + ' - ' + (Math.min(offset + this.viewport.getPageSize(), rows)) + ' ' + DimpText.of + ' ' + rows);
        } else {
            $('msgHeader').update(DimpText.nomessages);
        }
    },

    setFolderLabel: function(f)
    {
        var fid = this.getFolderId(f), unseen;
        var elt = $(fid);
        if (!elt || !elt.hasAttribute('u')) {
            return;
        }

        if (arguments.length == 2) {
            unseen = Math.max(arguments[1], 0);
            elt.writeAttribute('u', unseen);
        } else {
            unseen = parseInt(elt.readAttribute('u'));
        }

        $(fid + '_label').update((unseen > 0) ?
            new Element('STRONG').insert(elt.readAttribute('l')).insert('&nbsp;').insert(new Element('SPAN', { className: 'count', dir: 'ltr' }).insert('(' + unseen + ')')) :
            elt.readAttribute('l'));
    },

    getFolderId: function(f)
    {
        return 'fld' + decodeURIComponent(f).replace(/_/g,'__').replace(/[^0-9a-z\-_:\.]/gi, '_');
    },

    getSubFolderId: function(f)
    {
        return 'sub' + f;
    },

    // Preview pane functions
    togglePreviewPane: function()
    {
        this.showPreview = !this.showPreview;
        $('previewtoggle').setText(this.showPreview ? DimpText.hide_preview : DimpText.show_preview);
        if (this.showPreview) {
            $('msgList').removeClassName('msglistNoPreview');
        } else {
            $('msgList').addClassName('msglistNoPreview');
        }
        new Ajax.Request(DimpVar.URI_PREFS, { parameters: { app: 'dimp', pref: 'show_preview', value: this.showPreview ? 1 : 0 } });
        this.viewport.showSplitPane(this.showPreview);
        if (this.showPreview) {
            this.initPreviewPane();
        }
        this.updateTitle();
    },

    /* Folder list updates. */
    pollFolders: function()
    {
        // Reset poll folder counter.
        this.setPollFolders();
        var args = {};
        if (this.folder && $('dimpmain_folder').visible()) {
            args = this.viewport.addRequestParams({});
        }
        $('button_checkmail').setText('[' + DimpText.check + ']');
        this.doAction('PollFolders', args, [], this.pollFoldersCallback.bind(this));
    },

    pollFoldersCallback: function(r)
    {
        if (r.response) {
            r = r.response.poll;
            $H(r).keys().each(function(f) {
                this.setFolderLabel(f, r[f].u);
                if (this.viewport) {
                    this.viewport.setMetaData({ unseen: r[f].u }, f);
                }
            }.bind(this));
        }
        $('button_checkmail').setText(DimpText.getmail);
    },

    setPollFolders: function()
    {
        if (DimpVar.refresh_time) {
            if (this.pollPE) {
                this.pollPE.stop();
            }
            this.pollPE = new PeriodicalExecuter(this.pollFolders.bind(this), DimpVar.refresh_time);
        }
    },

    showOptions: function(params)
    {
        this.doAction('ShowOptions', params || { app: 'dimp' }, [], this.portalCallback.bind(this));
    },

    portalCallback: function(r)
    {
        if (!r.response) {
            return;
        }
        if (r.response.linkTags) {
            var head = $A(document.getElementsByTagName('HEAD')).first();
            r.response.linkTags.each(function(newLink) {
                var link = new Element('LINK', { type: 'text/css', rel: 'stylesheet', href: newLink.href });
                if (newLink.media) {
                    link.media = newLink.media;
                }
                head.appendChild(link);
            });
        }
        $('dimpmain_portal').update(r.response.portal);
        ToolTips.attachBehavior();
    },

    /* Search filter functions. */
    searchfilterStatus: function(e)
    {
        var f = $F('msgList_filter'),
            q = $('qoptions');
        var v = q.visible();

        if (v && !f) {
            this.searchfilterClear();
        } else if (!v && f) {
            $('sf_current').update(this.viewport.getMetaData('label'));
            this.setSearchfilterParams('to', 'msg');
            this.setSearchfilterParams('current', 'folder');
            q.show();
            this.filtertoggle = 1;
            this.viewport.onResize();
            this.updateTitle();
            this.resetSelected();
        } else {
            if (this.searchobserve) {
                clearTimeout(this.searchobserve);
            }
            this.searchobserve = this.viewport.runFilter.bind(this.viewport, f).delay(0.5);
        }
    },

    // false = onFocus, true = onBlur
    searchfilterOnAction: function(action)
    {
        if (!$('qoptions').visible()) {
            this.setFilterText(action);
        }
    },

    searchfilterClear: function(reset)
    {
        if (this.searchobserve) {
            clearTimeout(this.searchobserve);
            this.searchobserve = null;
        }
        if (!$('qoptions').visible()) {
            this.setFilterText(true);
            return;
        }
        $('qoptions').hide();
        this.filtertoggle = 2;
        this.setFilterText(false);
        this.viewport.onResize(reset);
        this.viewport.stopFilter(reset);
    },

    searchfilterClose: function(reset)
    {
        this.searchfilterClear(reset);
        this.setFilterText(true);
    },

    setFilterText: function(d)
    {
        var mf = $('msgList_filter');
        if (d) {
            mf.setValue(DimpText.search);
            mf.addClassName('msgFilterDefault');
        } else {
            mf.setValue('');
            mf.removeClassName('msgFilterDefault');
        }
    },

    // type = 'folder' or 'msg'
    setSearchfilterParams: function(id, type)
    {
        var c = (type == 'folder') ? this.sfiltersfolder : this.sfilters;
        c.keys().each(function(i) {
            $(i).writeAttribute('className', (id == c.get(i)) ? 'qselected' : '');
        });
    },

    updateSearchfilter: function(id, type)
    {
        this.setSearchfilterParams(id, type);
        this.viewport.runFilter();
    },

    addSearchfilterParams: function()
    {
        var sf = this.sfiltersfolder.keys().detect(function(s) {
            return $(s).hasClassName('qselected');
        });
        var sm = this.sfilters.keys().detect(function(s) {
            return $(s).hasClassName('qselected');
        });
        return { searchfolder: this.sfiltersfolder.get(sf), searchmsg: this.sfilters.get(sm) };
    },

    /* Add/remove mouse events on the fly.  Current types:
     *   'message', 'draft'  --  Message list rows
     *   'container', 'special', 'folder'  --  Folders
     *   'reply', 'forward', 'otheractions'  --  Message list buttons
     *   'contacts'  --  Linked e-mail addresses */
    addMouseEvents: function(id, type, offset, left, offsettype)
    {
        try {
            if (!this.drags.get(id)) {
                switch (type) {
                case 'draft':
                case 'message':
                    this.drags.set(id, new DraggableMail(id, DraggableMailOpts));
                    break;

                case 'container':
                case 'folder':
                    this.drags.set(id, new DraggableDimp(id, { delay: 100, ghosting: true, zindex: 15000, revert: true, reverteffect: function(e, t, l) { new Effect.Move(e, { x: -l, y: -t, duration: 0 }); }, scroll: 'normalfolders', scrollSensitivity: 15 }));
                    // Need this because add() will make any folder children
                    // 'relative', which breaks the folder list on IE.
                    if (Prototype.Browser.IE) {
                        $(id).undoPositioned();
                    }
                    break;

                case 'special':
                    // For purposes of the contextmenu, treat special folders
                    // like regular folders.
                    type = 'folder';
                    break;
                }
            }
            var opts = { offset: offset, left: left, onShow: this.onMenuShow };
            if (offsettype) {
                opts.offsettype = offsettype;
            }
            DMenu.addElement(id, 'ctx_' + type, opts);
        } catch (e) {}
    },

    removeMouseEvents: function(id)
    {
        if (this.drags.get(id)) {
            this.drags.unset(id).destroy();
        }
        DMenu.removeElement(id);
    },

    onMenuShow: function(ctx)
    {
        switch (ctx.ctx.substr(4)) {
        case 'folder':
            var folder = $(DMenu.element());
            if (folder.readAttribute('mbox') == 'INBOX') {
                $('ctx_folder_create', 'ctx_folder_rename', 'ctx_folder_delete').invoke('hide');
            } else if (DimpVar.fixed_folders.indexOf(folder.readAttribute('mbox')) != -1) {
                $('ctx_folder_rename', 'ctx_folder_delete').invoke('hide');
            } else {
                $('ctx_folder_create', 'ctx_folder_rename', 'ctx_folder_delete').invoke('show');
            }

            if (folder.hasAttribute('u')) {
                $('ctx_folder_poll').hide();
                $('ctx_folder_nopoll').show();
            } else {
                $('ctx_folder_poll').show();
                $('ctx_folder_nopoll').hide();
            }
            break;
        }
        return true;
    },

    /* Draggable/Droppable handlers. */
    folderHoverHandler: function(elt, folder, overlap)
    {
        if (overlap > 0.33 && overlap < 0.66 && $(this.getSubFolderId(folder.id))) {
            this.toggleSubFolder(folder.id, 'exp');
        }
    },

    folderDropHandler: function(elt, folder, e)
    {
        elt = $(elt);
        folder = $(folder);
        var foldername = folder.readAttribute('mbox'),
            ftype = folder.readAttribute('ftype');

        if (elt.hasClassName('folder')) {
            // Don't allow drag/drop to special folders and to child
            // folders (to prevent circular reference issues).
            if (ftype != 'special' && !folder.descendantOf(this.getSubFolderId(elt.id))) {
                this.doAction('RenameFolder', { old_name: elt.readAttribute('mbox'), new_parent: foldername, new_name: elt.readAttribute('l') }, [], this.folderCallback.bind(this));
            }
            e.stop();
        } else if (ftype != 'container') {
            var uids;

            if (this.selectedCount()) {
                // Dragging multiple selected messages.
                uids = this.getSelectedRows().pluck('uid');
            } else if (elt.readAttribute('mbox') != foldername) {
                // Dragging a single unselected message.
                uids = [ elt.id.substring(3) ];
            }

            // Don't allow drag/drop to the current folder.
            if (this.folder != foldername && uids) {
                this.viewport.updateFlag(uids, 'deletedmsg', true, true);
                this.doAction('MoveMessage', this.viewport.addRequestParams({ tofld: foldername }), uids, this.deleteCallback.bind(this));
            }
        }
    },

    /* Keydown event handler */
    keydownHandler: function(e)
    {
        // Currently, we only have message list handlers
        if (!$('dimpmain_folder').visible()) {
            return;
        }

        var kc = e.keyCode || e.charCode;
        switch (kc) {
        case Event.KEY_DELETE:
            var index;
            if (this.selectedCount() == 1) {
                index = this.selected.first();
                if (e.shiftKey) {
                    var r = this.viewport.getRowById([ index ]);
                    this.moveSelected(r.rownum + ((r.rownum == this.viewport.getMetaData('total_rows')) ? -1 : 1), true);
                }
                this.flag('deleted', index, this.folder);
            } else {
                this.flag('deleted');
            }
            e.stop();
            break;

        case Event.KEY_UP:
        case Event.KEY_DOWN:
            if (e.shiftKey) {
                if (this.lastrow) {
                    var uid;
                    switch (kc) {
                    case Event.KEY_UP:
                        if (this.lastrow != 1) {
                            uid = this.viewport.getRows(this.lastrow - 1, 1).first().uid;
                        }
                        break;

                    case Event.KEY_DOWN:
                        if (this.lastrow != this.viewport.getMetaData('total_rows')) {
                            uid = this.viewport.getRows(this.lastrow + 1, 1).first().uid;
                        }
                        break;
                    }

                    if (uid) {
                        this.viewport.positionById(uid);
                        this.shiftSelect(uid);
                    }
                }
            } else {
                this.moveSelected(kc == Event.KEY_UP ? -1 : 1);
            }
            e.stop();
            break;

        case Event.KEY_PAGEUP:
        case Event.KEY_PAGEDOWN:
            var ps = this.viewport.getPageSize() - 1;
            move = ps * (kc == Event.KEY_PAGEUP ? -1 : 1);
            if (this.selectedCount() == 1) {
                var co = this.viewport.currentOffset();
                var rowoff = this.viewport.getRowById([ this.selected.first() ]).rownum - 1;
                switch (kc) {
                case Event.KEY_PAGEUP:
                    if (co != rowoff) {
                        move = co - rowoff;
                    }
                    break;

                case Event.KEY_PAGEDOWN:
                    if ((co + ps) != rowoff) {
                        move = co + ps - rowoff;
                    }
                    break;
                }
            }
            this.moveSelected(move);
            e.stop();
            break;

        case Event.KEY_HOME:
        case Event.KEY_END:
            this.moveSelected(kc == Event.KEY_HOME ? 1 : this.viewport.getMetaData('total_rows'), true);
            e.stop();
            break;

        case Event.KEY_RETURN:
            if (!e.element().match('input')) {
                // Popup message window if single message is selected.
                var sr = this.getSelectedRows();
                if (sr.size() == 1) {
                    sr = sr.first();
                    this.msgWindow({ uid: sr.uid, folder: sr.mbox });
                }
                e.stop();
            }
            // Don't stop even by default since we may be in an INPUT
            // field, and we want the return to submit the form.
            break;

        case 65: // A
        case 97: // a
            if (e.ctrlKey) {
                this.selectAll();
                e.stop();
            }
            break;
        }
    },

    /* Handle rename folder actions. */
    renameFolder: function(folder)
    {
        if (Object.isUndefined(folder)) {
            return;
        }

        folder = $(folder);
        var n = this.createFolderForm(function(e) { this.folderAction(folder, e, 'rename'); return false; }.bindAsEventListener(this), DimpText.rename_prompt);
        n.down('input').setValue(folder.readAttribute('l'));
    },

    /* Handle insert folder actions. */
    createBaseFolder: function()
    {
        this.createFolderForm(function(e) { this.folderAction('', e, 'create'); return false; }.bindAsEventListener(this), DimpText.create_prompt);
    },

    createSubFolder: function(folder)
    {
        if (Object.isUndefined(folder)) {
            return false;
        }

        this.createFolderForm(function(e) { this.folderAction($(folder), e, 'createsub'); return false; }.bindAsEventListener(this), DimpText.createsub_prompt);
    },

    createFolderForm: function(action, text)
    {
        var n = new Element('FORM', { action: '#', id: 'RB_folder' }).insert(
                    new Element('P').insert(text)
                ).insert(
                    new Element('INPUT', { type: 'text', size: 15 })
                ).insert(
                    new Element('INPUT', { type: 'button', className: 'button', value: DimpText.ok }).observe('click', action)
                ).insert(
                    new Element('INPUT', { type: 'button', className: 'button', value: DimpText.cancel }).observe('click', RedBox.close.bind(RedBox))
                );

        RedBox.overlay = true;
        RedBox.onDisplay = Form.focusFirstElement.curry(n);
        RedBox.showHtml(n);
        return n;
    },

    folderAction: function(folder, e, mode)
    {
        RedBox.close();

        var form = e.findElement('form'), action, params;
        var val = $F(form.down('input'));

        if (val) {
            switch (mode) {
            case 'rename':
                if (folder.readAttribute('l') != val) {
                    action = 'RenameFolder';
                    params = { old_name: folder.readAttribute('mbox'),
                               new_name: val };
                }
                break;

            case 'create':
            case 'createsub':
                action = 'CreateFolder';
                params = { folder: val };
                if (mode == 'createsub') {
                    params.parent = folder.readAttribute('mbox');
                }
                break;
            }
            if (action) {
                this.doAction(action, params, [], this.folderCallback.bind(this));
            }
        }
    },

    /* Folder action callback functions. */
    folderCallback: function(r)
    {
        if (!r.response) {
            return;
        }

        r.response.d.each(function(i) {
            this.deleteFolder(i);
        }.bind(this));

        r.response.c.each(function(i) {
            this.changeFolder(i);
        }.bind(this));

        r.response.a.each(function(i) {
            this.createFolder(i);
        }.bind(this));
    },

    deleteCallback: function(r)
    {
        this.msgListLoading(false);

        if (!r.response ||
            !r.response.uids ||
            !r.response.remove ||
            !r.response.folder == this.folder) {
            return;
        }

        var uids = r.response.uids.parseRangeString();
        this.viewport.remove(uids, r.response.cacheid);
        if (uids.numericSort().toString() == this.selected.numericSort().toString()) {
            this.resetSelected();
        }
    },

    emptyFolderCallback: function(r)
    {
        if (r.response.mbox) {
            if (this.folder == r.response.mbox) {
                this.viewport.reload();
                this.clearPreviewPane();
            }
            this.setFolderLabel(r.response.mbox, 0);
        }
    },

    flagAllCallback: function(r)
    {
        if (r.response.mbox) {
            this.setFolderLabel(r.response.mbox, r.response.u);
        }
    },

    folderLoadCallback: function(r)
    {
        if (!r.response) {
            return;
        }

        this.folderCallback(r);

        var fs = $('foldersSidebar');
        fs.observe('click', this.handleFolderMouseEvent.bindAsEventListener(this, 'click'));
        fs.observe('mouseover', this.handleFolderMouseEvent.bindAsEventListener(this, 'over'));
        fs.observe('mouseout', this.handleFolderMouseEvent.bindAsEventListener(this, 'out'));

        $('foldersLoading').hide();
        fs.show();

        var mh = $('normalfolders').getStyle('max-height');

        // Fix for IE6 - which doesn't support max-height.  We need to search
        // for height: 0px instead (comment in IE 6 CSS explains this is
        // needed for auto sizing).
        if (Prototype.Browser.IE && Object.isUndefined(mh)) {
            mh = $('normalfolders').getStyle('height') == '0px' ? true : null;
        }

        if (mh !== null) {
            this.sizeFolderlist();
            Event.observe(window, 'resize', this.sizeFolderlist.bind(this));
        }
    },

    handleFolderMouseEvent: function(e, action)
    {
        var elt = e.element(), li;
        if (elt.nodeType == 3) {
            elt = elt.parentNode;
        }
        if (elt.hasClassName('folder') || elt.hasClassName('custom')) {
            li = elt;
        } else {
            li = elt.up('.folder');
            if (!li) {
                li = elt.up('.custom');
                if (!li) {
                    return;
                }
            }
        }

        var type = li.readAttribute('ftype');

        switch (action) {
        case 'over':
            li.addClassName('over');
            if (type) {
                this.addMouseEvents(li.id, type);
            }
            break;

        case 'out':
            li.removeClassName('over');
            break;

        case 'click':
            if (elt.hasClassName('exp') || elt.hasClassName('col')) {
                this.toggleSubFolder(li.id, 'tog');
            } else {
                switch (type) {
                case 'container':
                    e.stop();
                    break;

                case 'folder':
                case 'special':
                    return this.go('folder:' + li.readAttribute('mbox'));
                    break;
                }
            }
            break;
        }
    },

    toggleSubFolder: function(base, mode)
    {
        base = $(base);
        var s = $(this.getSubFolderId(base.id));
        if (mode == 'tog' ||
            (mode == 'exp' && !s.visible()) ||
            (mode == 'col' && s.visible())) {
            base.firstDescendant().className = s.toggle().visible() ? 'col' : 'exp';
        }
    },

    // Folder actions.
    // For format of the ob object, see DIMP::_createFolderElt().
    createFolder: function(ob)
    {
        var a, div,
            fid = this.getFolderId(ob.m),
            insert_node = 0,
            li, ll,
            mbox = decodeURIComponent(ob.m),
            parent, submbox;

        if (ob.s) {
            parent = $('specialfolders');
        } else {
            parent = $(this.getSubFolderId(this.getFolderId(ob.pa)));
            if (parent) {
                if (parent.firstDescendant().match('ul')) {
                    parent = parent.firstDescendant();
                }
            } else {
                parent = $('normalfolders');
            }
        }

        li = new Element('LI', { className: 'folder', id: fid, l: ob.l, mbox: mbox, ftype: ((ob.co) ? 'container' : ((ob.s) ? 'special' : 'folder')) });

        div = new Element('DIV', { className: ob.cl, id: fid + '_div' });
        if (ob.i) {
            div.update(ob.i);
        }
        if (ob.ch) {
            div.className = 'exp';
        }
        li.appendChild(div);

        a = new Element('A', { id: fid + '_label', nicetitle: ob.m }).insert(ob.l);
        li.appendChild(a);

        // Now walk through the parent <ul> to find the right place to
        // insert the new folder.
        submbox = $(this.getSubFolderId(fid));
        if (submbox) {
            parent.insertBefore(li, submbox);
            // If an expanded parent mailbox was deleted, we need to toggle the
            // icon accordingly.
            if (submbox.visible()) {
                div.removeClassName('exp');
                div.addClassName('col');
            }
        } else {
            ll = mbox.toLowerCase();
            parent.immediateDescendants().detect(function(node) {
                var nodembox = node.readAttribute('mbox');
                if (nodembox && (!ob.s || nodembox != 'INBOX')) {
                    if (ll < nodembox.toLowerCase()) {
                        insert_node = 1;
                        parent.insertBefore(li, node);
                        return true;
                    }
                }
            });

            if (!insert_node) {
                parent.appendChild(li);
            }
        }

        // Make the new folder a drop target.
        Droppables.add(fid, { hoverclass: 'drop', onHover: this.folderHoverHandler.bind(this), onDrop: this.folderDropHandler.bind(this), scroll: true, overlap: 'vertical' });

        // Need this because add() will make the children 'relative',
        // which breaks the folder list on IE.
        if (Prototype.Browser.IE) {
            $(fid).undoPositioned();
        }

        // Check for unseen messages
        if (ob.po) {
            li.writeAttribute('u', '');
            this.setFolderLabel(mbox, ob.u);
        }

        // Make sure the sub<mbox> ul is created if necessary.
        if (ob.ch && !submbox) {
            parent.insertBefore(new Element('LI', { className: 'subfolders', id: this.getSubFolderId(fid) }).insert(new Element('UL')).hide(), li.nextSibling);
        }
    },

    deleteFolder: function(folder)
    {
        var f = decodeURIComponent(folder);
        if (this.folder == f) {
            this.go('folder:INBOX');
        }

        var fid = this.getFolderId(folder);
        this.deleteFolderElt(fid, true);

        Effect.Fade(fid, { afterFinish: function(effect) {
            try {
                this.remove_gc.push(effect.element.remove());
            } catch (e) {
                this.debug('deleteFolder', e);
            }
        }.bind(this) });
    },

    changeFolder: function(ob)
    {
        var fid = this.getFolderId(ob.m), fdiv = $(fid + '_div');
        this.deleteFolderElt(fid, !ob.ch);
        var oldexpand = fdiv.hasClassName('col');
        if (ob.co && this.folder == ob.m) {
            this.go('folder:INBOX');
        }
        $(fid).remove();
        this.createFolder(ob);
        if (ob.ch && oldexpand) {
            fdiv.removeClassName('exp');
            fdiv.addClassName('col');
        }
    },

    deleteFolderElt: function(fid, sub)
    {
        this.remove_gc = this.remove_gc.concat($(fid, fid + '_div', fid + '_label'));
        if (sub) {
            var submbox = $(this.getSubFolderId(fid));
            if (submbox) {
                submbox.remove();
            }
        }
        Droppables.remove(fid);
        this.removeMouseEvents(fid);
        this.remove_gc.push(fid);
        if (this.viewport) {
            this.viewport.deleteView(fid);
        }
    },

    sizeFolderlist: function()
    {
        var fl = $('normalfolders');
        fl.setStyle({ height: (document.viewport.getHeight() - fl.cumulativeOffset()[1] - 10) + 'px' });
    },

    /* Flag actions for message list. */
    flag: function(action, index, folder)
    {
        var args, mbox, rows, uids = [];

        if (index && folder) {
            rows = [{ id: index, uid: index, mbox: folder }];
            mbox = folder;
        } else {
            rows = this.getSelectedRows();
            mbox = this.folder;
        }

        switch (action) {
        case 'allUnseen':
        case 'allSeen':
            this.doAction((action == 'allUnseen') ? 'MarkFolderUnseen' : 'MarkFolderSeen', { folder: folder }, [], this.flagAllCallback.bind(this));
            if (folder == this.folder) {
                this.viewport.updateFlag(this.viewport.getRows(1, this.viewport.getBufferSize()).pluck('uid'), 'unseen', action == 'allUnseen', true, mbox);
            }
            break;

        case 'deleted':
        case 'undeleted':
        case 'spam':
        case 'ham':
        case 'blacklist':
        case 'whitelist':
            if (!rows.size()) {
                break;
            }
            args = this.viewport.addRequestParams({});
            uids = rows.pluck('uid');
            if (action == 'deleted' || action == 'undeleted') {
                this.viewport.updateFlag(uids, 'deletedmsg', action == 'deleted', true, mbox);
            }

            if (action == 'undeleted') {
                this.doAction('UndeleteMessage', args, uids);
            } else {
                var actionCall = { deleted: 'DeleteMessage', spam: 'ReportSpam', ham: 'ReportHam', blacklist: 'Blacklist', whitelist: 'Whitelist' };
                // This needs to be synchronous Ajax if we are calling from a
                // popup window because Mozilla will not correctly call the
                // callback function if the calling window has been closed.
                this.doAction(actionCall[action], args, uids, this.deleteCallback.bind(this), { asynchronous: !(index && folder) });

                // If reporting spam, to indicate to the user that something is
                // happening (since spam reporting may not be instantaneous.
                if (action == 'spam' || action == 'ham') {
                    this.msgListLoading(true);
                }
            }
            break;

        case 'unseen':
        case 'seen':
            if (!rows.size()) {
                break;
            }
            args = { folder: this.folder, messageFlag: '-seen' };
            var unseenstatus = 1;
            if (action == 'seen') {
                unseenstatus = 0;
                args.messageFlag = 'seen';
            }
            rows.each(function(s) {
                if (this.updateUnseenUID(s.uid, s.mbox, unseenstatus)) {
                    uids.push(s.uid);
                }
            }.bind(this));

            if (uids.length) {
                this.doAction('MarkMessage', args, uids);
            }
            break;

        case 'flagged':
        case 'clear':
            if (!rows.size()) {
                break;
            }
            args = {
                folder: this.folder,
                messageFlag: ((action == 'flagged') ? 'flagged' : '-flagged')
            };
            this.viewport.updateFlag(rows.pluck('uid'), 'flagged', action == 'flagged', true, mbox);
            this.doAction('MarkMessage', args, rows.pluck('uid'));
            break;

        case 'answered':
            uids = rows.pluck('uid');
            this.viewport.updateFlag(uids, 'answered', true, true, mbox);
            this.viewport.updateFlag(uids, 'flagged', false, true, mbox);
            break;
        }

        return false;
    },

    /* Miscellaneous folder actions. */
    purgeDeleted: function()
    {
        this.doAction('PurgeDeleted', this.viewport.addRequestParams({}), [], this.deleteCallback.bind(this));
    },

    modifyPollFolder: function(folder, add)
    {
        this.doAction('ModifyPollFolder', { folder: folder, add: (add) ? 1 : 0 }, [], this.modifyPollFolderCallback.bind(this));
    },

    modifyPollFolderCallback: function(r)
    {
        if (!r.response) {
            return;
        }

        r = r.response;
        var f = r.folder, p = { response: { poll: {} } };
        var fid = $(this.getFolderId(f));

        if (r.add) {
            p.response.poll[f] = { u: r.poll.u };
            fid.writeAttribute('u', 0);
        } else {
            p.response.poll[f] = { u: 0 };
        }

        this.pollFoldersCallback(p);

        if (!r.add) {
            fid.removeAttribute('u');
        }
    },

    /* Add a popdown menu to a dimpactions button. */
    addPopdown: function(bid, ctx, offsettype)
    {
        var bidelt = $(bid),
            img = $($('popdown_img').firstDescendant().cloneNode(false));
        img.id = bid + '_img';
        var parentelt = bidelt.up();
        parentelt.appendChild(img);
        this.addMouseEvents(img.id, ctx, parentelt, true, offsettype);
    },

    /* Add dropdown menus to address links. */
    buildAddressLinks: function(id)
    {
        var acount = 0;
        $(id).select('.address').each(function(e) {
            e.id = 'addr' + acount++;
            e.observe('mouseover', function() { this.removeMouseEvents(e.id); this.addMouseEvents(e.id, 'contacts', e, true, 'positioned'); }.bind(this));
        }.bind(this));
    },

    /* Utility functions. */
    toggleBlock: function(block, toggle, hidetxt, showtxt, style)
    {
        block.toggle();

        var link = new Element('A', { className: 'widget', style: style || '' }).insert('[' + (block.visible() ? hidetxt : showtxt) + ']');
        link.observe('click', this.toggleBlock.bind(this, block, toggle, hidetxt, showtxt, style));

        if (toggle.firstChild) {
            $(toggle.firstChild).remove();
        }
        toggle.appendChild(link);

        return false;
    },

    msgListLoading: function(show)
    {
        if (this.fl_visible != show) {
            this.fl_visible = show;
            if (show) {
                Effect.Appear('folderLoading', { duration: 0.2 });
            } else {
                Effect.Fade('folderLoading', { duration: 0.2 });
            }
        }
    }
};

/* Stuff to do after window loads. */
document.observe('dom:loaded', function() {
    /* Set up context menu handler. */
    if (typeof ContextSensitive != 'undefined') {
        DMenu = new ContextSensitive();
    }

    /* Don't do additional onload stuff if we are in a popup. We need a
     * try/catch block here since, if the page was loaded by an opener
     * out of this current domain, this will throw an exception. */
    try {
        if (parent.opener &&
            parent.opener.location.host == window.location.host &&
            parent.opener.DimpCore) {
            baseWindow = parent.opener.baseWindow || parent.opener;
            return;
        } else if (!$('dimpLoading')) {
            /* We have directly accessed a popup page (e.g. compose.php,
             * message.php). */
            return;
        }
    } catch (e) {}

    /* Create the folder list. Any pending notifications will be caught via
     * the return from this call. */
    this.doAction('ListFolders', {}, [], this.folderLoadCallback.bind(this));

    /* Catch shift-selects in IE - only needed for message list page. */
    if (Prototype.Browser.IE) {
        document.observe('selectstart', function(e) { if (DimpVar.is_ie6 || !e.element().descendantOf('previewPane')) { e.stop(); } });
    }

    /* Remove unneeded buttons. */
    if (!DimpVar.spam_reporting) {
        this.buttons = this.buttons.without('button_spam');
    }
    if (!DimpVar.ham_reporting) {
        this.buttons = this.buttons.without('button_ham');
    }

    /* Remove unneeded search folders. */
    if (!DimpVar.search_all) {
        this.sfiltersfolder.unset('sf_all');
    }

    /* Check for new mail. */
    this.setPollFolders();

    $('dimpLoading').hide();
    $('dimpPage').show();

    if (Horde.dhtmlHistory.initialize()) {
        Horde.dhtmlHistory.addListener(this.show.bind(this));
    }

    /* Initialize the starting page if necessary. addListener() will have
     * already fired if there is a current location so only do a show() call
     * if there is no current location. */
    if (!Horde.dhtmlHistory.getCurrentLocation()) {
        if (DimpVar.login_view == 'inbox') {
            this.go('folder:INBOX');
        } else {
            this.show('portal');
            if (DimpVar.background_inbox) {
                this.folder = 'INBOX';
                this.loadFolder(true);
            }
        }
    }

    /* Load quota bar. */
    if (DimpVar.show_quota) {
        this.checkQuota();
        if (DimpVar.refresh_time) {
            new PeriodicalExecuter(this.checkQuota.bind(this), DimpVar.refresh_time);
        }
    }

    /* Bind key shortcuts. */
    document.observe('keydown', this.keydownHandler.bind(this));

    /* Init garbage collection function - runs every 10 seconds. */
    new PeriodicalExecuter(function() {
        if (this.remove_gc.size()) {
            try {
                this.remove_gc.splice(0, 75).invoke('stopObserving');
            } catch (e) {
                this.debug('remove_gc[].stopObserving', e);
            }
        }
    }.bind(this), 10);

    /* In preview pane, clear any context menus on mousewheel scroll - it is
     * the only case we can not catch elsewhere. */
    $('previewPane').observe(Prototype.Browser.IE ? 'mousewheel' : 'DOMMouseScroll', DMenu.close);

    /* Add popdown menus. */
    this.addPopdown('button_reply', 'reply', 'cumulative');
    DMenu.disable('button_reply_img', true, true);
    this.addPopdown('button_forward', 'forward', 'cumulative');
    DMenu.disable('button_forward_img', true, true);
    this.addPopdown('button_other', 'otheractions', 'cumulative');
}.bind(DimpCore));

/* Helper methods for setting/getting element text without mucking
 * around with multiple TextNodes. */
Element.addMethods({
    setText: function(element, text)
    {
        if (!(element = $(element))) {
            return;
        }
        var t = 0;
        $A(element.childNodes).each(function(node) {
            if (node.nodeType == 3) {
                if (t++) {
                    node.remove();
                } else {
                    node.nodeValue = text;
                }
            }
        });
    },

    getText: function(element, recursive)
    {
        if (!(element = $(element))) {
            return;
        }
        var text = '';
        $A(element.childNodes).each(function(node) {
            if (node.nodeType == 3) {
                text += node.nodeValue;
            } else if (recursive && node.childNodes) {
                text += $(node).getText(true);
            }
        });
        return text;
    }
});

/* Create some utility functions. */
Object.extend(Array.prototype, {
    toRangeString: function()
    {
        var first = null, last = null, out = [];
        this.numericSort().each(function(k) {
            if (first) {
                if (last + 1 == k) {
                    last = k;
                } else {
                    out.push(first + (last == first ? '' : (':' + last)));
                    first = last = k;
                }
            } else {
                first = last = k;
            }
        });

        out.push(first + (last == first ? '' : (':' + last)));
        return out.join(',');
    },

    numericSort: function()
    {
        return this.sort(function(a,b) { return a - b; });
    }
});

Object.extend(String.prototype, {
    parseRangeString: function()
    {
        var mlist = [], str = this.strip();
        if (!str.blank()) {
            str.split(',').each(function(e) {
                var r = e.split(':');
                if (r.size() == 1) {
                    mlist.push(parseInt(e));
                } else {
                    mlist = mlist.concat($A($R(parseInt(r[0]), parseInt(r[1]))));
                }
            });
        }
        return mlist;
    },

    // We define our own version of evalScripts() to make sure that all
    // scripts are running in the same scope and that all functions are
    // defined in the global scope. This is not the case when using
    // prototype's evalScripts().
    evalScripts: function()
    {
        var func, scripts = this.extractScripts(), re = /function\s+([^\s(]+)/g;
        for (var i = 0, length = scripts.length; i < length; ++i) {
            eval(scripts[i]);
            while (func = re.exec(scripts[i])) {
                window[func[1]] = eval(func[1]);
            }
        }
    }
});

Hash.addMethods({
    flip: function()
    {
        var h = $H();
        this.each(function(l) {
            h.set(l.value, l.key);
        });
        return h;
    }
});

// Workaround for prototypejs bug (as of 1.6.0):
//   http://dev.rubyonrails.org/ticket/10561
DimpCore.isLeftClick = function(e) {
    if (Prototype.Browser.IE) {
        return (e.button == 0 && e.type != 'contextmenu');
    } else {
        return e.isLeftClick();
    }
};

/** Functions overriding IMP/Horde JS functions. **/

/* We need to replace the IMP javascript for this function with code that
 * calls the correct DIMP functions. */
function popup_imp(url, w, h, args)
{
    DimpCore.compose('new', args.toQueryParams().toObject());
}

/* Necessary includes for toggle divs. */
function toggleQuoteBlock(id, lines)
{
    DimpCore.toggleBlock($('qb_' + id), $('qt_' + id), DimpText.hidetext, DimpText.showtext + ' - ' + lines + ' ' + DimpText.lines, 'fontSize:70%;');
}

function toggleAddressesBlock(field, count)
{
    DimpCore.toggleBlock($('ab_' + field), $('at_' + field), DimpText.hideaddr, DimpText.showaddr + ' - ' + count + ' ' + DimpText.recips);
}

/* For attachment viewing to work, replaces the function from
 * horde/templates/contents/open_view_win.js. */
function view(url, partid)
{
    window.open(url, ++DimpCore.compose_id + partid.replace(/\W/g, '_'), 'menubar=yes,toolbar=no,location=no,status=no,scrollbars=yes,resizable=yes');
}
