/**
 * DimpSlider.js - Extensions to the stock Scriptaculous Slider library
 * which allows us to make the slider act more like a scrollbar.
 *
 * $Horde: dimp/js/src/DimpSlider.js,v 1.6.2.3 2008/01/06 03:46:36 slusarz Exp $
 *
 * Copyright 2007-2008 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

var DimpSlider = Class.create(Control.Slider, {

    // Changed from original by calling the onClickTrack callback (if present)
    // when we detect a click on the track.
    startDrag: function(event)
    {
        var e = Event.extend(event);

        if (!e.isLeftClick()) {
            return;
        }

        e.stop();

        if (this.disabled){
            return;
        }

        this.active = true;

        var handle = e.element(),
            pointer  = [ e.pointerX(), e.pointerY() ];

        if (handle == this.track) {
            var offsets = this.track.cumulativeOffset();
            this.event = e;
            var val = (this.isVertical() ? pointer[1] - offsets[1] : pointer[0] - offsets[0]) - (this.handleLength / 2);
            if (this.options.onClickTrack) {
                this.options.onClickTrack(val, this);
            } else {
                this.setValue(this.translateToValue(val));
                this.setOffsets(pointer);
            }
        } else {
            // Find the handle (prevents issues with Safari)
            while ((this.handles.indexOf(handle) == -1) && handle.parentNode) {
                handle = handle.parentNode;
            }

            if (this.handles.indexOf(handle) != -1) {
                this.activeHandle = handle;
                this.activeHandleIdx = this.handles.indexOf(this.activeHandle);
                this.updateStyles();

                this.setOffsets(pointer);
            }
        }
    },

    setOffsets: function(p)
    {
        var offsets = this.activeHandle.cumulativeOffset();
        this.offsetX = p[0] - offsets[0];
        this.offsetY = p[1] - offsets[1];
    },

    updateTrackLength: function()
    {
        this.trackLength = this.maximumOffset() - this.minimumOffset();
    },

    setHandleLength: function(l)
    {
        this.handleLength = l;
    },

    getValue: function(h)
    {
        return this.values[h || 0];
    },

    setScrollPosition: function(offset)
    {
        // Safari 2 fix - trackLength can be NaN here
        if (isNaN(this.trackLength)) {
            this.trackLength = this.maximumOffset() - this.minimumOffset();
        }
        this.setValue(offset);
        this.offsetX = this.offsetY = 0;
    },

    clearClick: function()
    {
        this.active = false;
    }

});
