<?php
/**
 * $Horde: dimp/lib/DIMP.php,v 1.110.2.3 2008/01/18 20:19:12 slusarz Exp $
 *
 * Copyright 2005-2008 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */
class DIMP {

    /**
     * Output a dimp-style action (menubar) link.
     *
     * @param string $url      The URL.
     * @param string $title    The title string.
     * @param string $tooltip  Tooltip text to use.
     * @param string $icon     The icon filename to display.
     * @param string $onclick  Any onclick javascript to perform.
     * @param string $id       The DOM ID of the link.
     * @param string $class    The CSS classname to use for the link.
     * @param string $app      The application to load the icon from.
     *
     * @return string  An HTML link to $url.
     */
    function actionButton($url, $title, $tooltip, $icon = '', $onclick = '',
                          $id = null, $class = '', $app = 'dimp')
    {
        $ak = Horde::getAccessKey($title);

        return Horde::link($url, $tooltip, $class, '', $onclick, '', $ak, empty($id) ? array() : array('id' => $id)) .
            (($icon) ? Horde::img($icon, $title, '', $GLOBALS['registry']->getImageDir($app)) : '') .
            Horde::highlightAccessKey($title, $ak) . '</a>';
    }

    /**
     * Output everything up to but not including the <body> tag.
     *
     * @param string $title   The title of the page.
     * @param array $scripts  Any additional scripts that need to be loaded.
     *                        Each entry contains the three elements necessary
     *                        for a Horde::addScriptFile() call.
     */
    function header($title, $scripts = array())
    {
        // Need to include script files before we start output
        Horde::addScriptFile('prototype.js', 'horde', true);

        Horde::addScriptFile('effects.js', 'imp', true);
        Horde::addScriptFile('controls.js', 'imp', true);
        Horde::addSCriptFile('unblockImages.js', 'imp', true);

        Horde::addScriptFile('DimpCore.js', 'dimp', true);

        // Add other scripts now
        foreach ($scripts as $val) {
            call_user_func_array(array('Horde', 'addScriptFile'), $val);
        }

        $page_title = $GLOBALS['registry']->get('name');
        if (!empty($title)) {
            $page_title .= ' :: ' . $title;
        }

        if (isset($GLOBALS['language'])) {
            header('Content-type: text/html; charset=' . NLS::getCharset());
            header('Vary: Accept-Language');
        }

        echo '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "DTD/xhtml1-transitional.dtd">' . "\n" .
             (!empty($GLOBALS['language']) ? '<html lang="' . strtr($GLOBALS['language'], '_', '-') . '"' : '<html') . ">\n".
             "<head>\n" .
             '<title>' . htmlspecialchars($page_title) . "</title>\n" .
             '<link href="' . $GLOBALS['registry']->getImageDir() . "/favicon.ico\" rel=\"SHORTCUT ICON\" />\n".
             '<script type="text/javascript">//<![CDATA[' . "\n" .
             DIMP::_includeDIMPJSVars() .
             "\n//]]></script>\n";

        DIMP::_includeScriptFiles();
        DIMP::_includeStylesheetFiles();
        IMP::outputInlineScript();

        echo "</head>\n";
    }

    /**
     * Outputs the javascript code which defines all javascript variables
     * that are dependant on the local user's account.
     *
     * @private
     */
    function _includeDIMPJSVars()
    {
        global $browser, $conf, $dimp_conf, $dimp_prefs, $prefs, $registry;

        $charset = NLS::getCharset();
        $compose_mode = (strstr($_SERVER['PHP_SELF'], 'compose.php') || strstr($_SERVER['PHP_SELF'], 'message.php'));
        $dimp_webroot = $registry->get('webroot', 'dimp');
        $horde_webroot = $registry->get('webroot', 'horde');

        /* Application links. */
        $app_urls = array();

        foreach (DIMP::menuList() as $app) {
            $app_urls[$app] = Horde::url($registry->getInitialPage($app), true);
        }

        /* Variables used in core javascript files. */
        $var = array(
            'URI_DIMP_INBOX' => Horde::url($dimp_webroot . '/index.php', true, -1),
            'URI_IMP' => Horde::url($dimp_webroot . '/imp.php', true, -1),
            'URI_PREFS' => Horde::url($horde_webroot . '/services/prefs/', true),
            'SESSION_ID' => defined('SID') ? SID : '',

            'app_urls' => $app_urls,
            'options_url' => Horde::url($dimp_webroot . '/options.php'),
            'timeout_url' => Auth::addLogoutParameters($horde_webroot . '/login.php', AUTH_REASON_SESSION),
            'message_url' => Horde::url($dimp_webroot . '/message.php'),
            'compose_url' => Horde::url($dimp_webroot . '/compose.php'),

            'sortfrom' => SORTFROM,
            'sortsubject' => SORTSUBJECT,
            'sortthread' => SORTTHREAD,
            'sortdate' => SORTDATE,
            'sortarrival' => SORTARRIVAL,
            'sortsize' => SORTSIZE,

            'popup_width' => 820,
            'popup_height' => 610,

            'spam_folder' => IMP::folderPref($prefs->getValue('spam_folder'), true),
            'spam_reporting' => (int) !empty($conf['spam']['reporting']),
            'ham_reporting' => (int) !empty($conf['notspam']['reporting']),
            'refresh_time' => (int) $prefs->getValue('refresh_time'),
            'search_all' => (int) !empty($conf['search']['search_all']),

            'fixed_folders' => empty($conf['server']['fixed_folders'])
                ? array()
                : array_map(array('DIMP', '_appendedFolderPref'), $conf['server']['fixed_folders']),

            'name' => $registry->get('name', 'dimp'),

            'preview_pref' => (bool)$dimp_prefs->getValue('show_preview'),

            'is_ie6' => ($browser->isBrowser('msie') && ($browser->getMajor() < 7)),

            'buffer_pages' => intval($dimp_conf['viewport']['buffer_pages']),
            'limit_factor' => intval($dimp_conf['viewport']['limit_factor']),
            'viewport_wait' => intval($dimp_conf['viewport']['viewport_wait']),
            'login_view' => $dimp_prefs->getValue('login_view'),
            'background_inbox' => !empty($dimp_conf['viewport']['background_inbox']),

            'show_quota' => (isset($_SESSION['imp']['quota']) && is_array($_SESSION['imp']['quota'])),

            // Turn debugging on?
            'debug' => !empty($dimp_conf['js']['debug']),
        );

        /* Gettext strings used in core javascript files. */
        $gettext = array_map('addslashes', array(
            'portal' => _("Portal"),
            'prefs' => _("User Options"),
            'search' => _("Search"),
            'resfound' => _("results found"),
            'messages' => _("Messages"),
            'of' => _("of"),
            'nomessages' => _("No Messages"),
            'hidetext' => _("Hide Quoted Text"),
            'showtext' => _("Show Quoted Text"),
            'lines' => _("lines"),
            'hideaddr' => _("Hide Addresses"),
            'showaddr' => _("Show Addresses"),
            'recips' => _("recipients"),
            'ok' => _("Ok"),
            'cancel' => _("Cancel"),
            'loading' => _("Loading..."),
            'totext' => _("To"),
            'from' => _("From"),
            'thread' => _("Thread"),
            'subject' => _("Subject"),
            'datesort' => _("Date"),
            'arrival' => _("Arrival"),
            'check' => _("Checking..."),
            'getmail' => _("Get Mail"),
            'ajax_timeout' => _("There has been no contact with the remote server for several minutes. The server may be temporarily unavailable or network problems may be interrupting your session. You will not see any updates until the connection is restored."),
            'ajax_recover' => _("The connection to the remote server has been restored."),
            'listmsg_wait' => _("The server is still generating the message list."),
            'listmsg_timeout' => _("The server was unable to generate the message list. Please try again later."),
            'popup_block' => _("A popup window could not be opened. Your browser may be blocking popups."),
            'hide_preview' => _("Hide Preview"),
            'show_preview' => _("Show Preview"),
            'rename_prompt' => _("Rename folder to:"),
            'create_prompt' => _("Create folder:"),
            'createsub_prompt' => _("Create subfolder:"),
        ));

        require_once 'Horde/Serialize.php';
        $code = 'var DimpVar = ' . Horde_Serialize::serialize($var, SERIALIZE_JSON, $charset) . ',' .
                'DimpText = ' . Horde_Serialize::serialize($gettext, SERIALIZE_JSON, $charset) . ';';

        if (!$compose_mode) {
            return $code;
        }

        /* Variables used in compose page. */
        $compose_cursor = $GLOBALS['prefs']->getValue('compose_cursor');
        $var_compose = array(
            'rte_avail' => $browser->hasFeature('rte'),
            'cc' => (bool) $prefs->getValue('compose_cc'),
            'bcc' => (bool) $prefs->getValue('compose_bcc'),
            'attach_limit' => ($conf['compose']['attach_count_limit'] ? intval($conf['compose']['attach_count_limit']) : -1),
            'close_draft' => $prefs->getValue('close_draft'),
            'js_editor' => $dimp_prefs->getValue('jseditor'),
            'compose_cursor' => ($compose_cursor ? $compose_cursor : 'top'),
        );

        /* Gettext strings used in compose page. */
        $gettext_compose = array_map('addslashes', array(
            'cancel' => _("Cancelling this message will permanently discard its contents and will delete auto-saved drafts.\nAre you sure you want to do this?"),
            'fillform' => _("You have already changed the message body, are you sure you want to drop the changes?"),
            'remove' => _("remove"),
            'uploading' => _("Uploading..."),
            'attachment_limit' => _("The attachment limit has been reached."),
            'sending' => _("Sending..."),
            'saving' => _("Saving..."),
        ));

        $code .= 'DimpVar_compose = ' . Horde_Serialize::serialize($var_compose, SERIALIZE_JSON, $charset) . ',' .
                 'DimpText_compose = ' . Horde_Serialize::serialize($gettext_compose, SERIALIZE_JSON, $charset) . ';';

        return $code;
    }

    /**
     * Return an appended IMP folder string
     */
    function _appendedFolderPref($folder)
    {
        return IMP::folderPref($folder, true);
    }

    /**
     * Outputs the necessary script tags, honoring local configuration
     * choices as to script caching.
     *
     * @private
     */
    function _includeScriptFiles()
    {
        if (empty($GLOBALS['conf']['server']['cachejs']) ||
            empty($GLOBALS['conf']['cache']['driver'])) {
            Horde::includeScriptFiles();
            return;
        }

        $js_tocache = $js_force = array();
        $mtime = 0;

        $s_list = Horde::listScriptFiles();
        foreach ($s_list as $app => $files) {
            foreach ($files as $file) {
                if ($file['d'] && ($file['f'][0] != '/')) {
                    $js_tocache[$file['p'] . $file['f']] = false;
                    if (!empty($GLOBALS['conf']['server']['cachejsparams']['mtime'])) {
                        $mtime = max($mtime, filemtime($file['p'] . $file['f']));
                    }
                } else {
                    if (!$file['d'] &&
                        ($app == 'dimp') &&
                        ($file['f'] == 'mailbox.js')) {
                        // Special dimp case: we keep mailbox.js in templates
                        // not for purposes of running PHP scripts in the
                        // file, but for ease of templating.  Thus, this file
                        // is OK to include inline.
                        $js_tocache[$file['p'] . $file['f']] = true;
                        if (!empty($GLOBALS['conf']['server']['cachejsparams']['mtime'])) {
                            $mtime = max($mtime, filemtime($file['p'] . $file['f']));
                        }
                    } else {
                        $js_force[] = $file['u'];
                    }
                }
            }
        }

        require_once 'Horde/Cache.php';
        require_once DIMP_BASE . '/lib/version.php';
        $cache = &Horde_Cache::singleton($GLOBALS['conf']['cache']['driver'], Horde::getDriverConfig('cache', $GLOBALS['conf']['cache']['driver']));

        $sig = md5(serialize($s_list) . $mtime . DIMP_VERSION);
        $exists = $cache->exists($sig, empty($GLOBALS['conf']['server']['cachejsparams']['lifetime']) ? 0 : $GLOBALS['conf']['server']['cachejsparams']['lifetime']);

        if (!$exists) {
            $out = '';
            foreach ($js_tocache as $key => $val) {
                if ($val) {
                    // Minify these files a bit by removing newlines and
                    // comments.
                    $out .= preg_replace(array('/\n+/', '/\/\*.*?\*\//'), array('', ''), file_get_contents($key));
                } else {
                    // Separate JS files with a newline since some compressors
                    // may strip trailing terminators.
                    $out .= file_get_contents($key) . "\n";
                }
            }
            $cache->set($sig, $out);
        }

        foreach (array_merge(array(DIMP::getCacheURL('js', $sig)), $js_force) as $val) {
            echo '<script type="text/javascript" src="' . $val . '"></script>' . "\n";
        }
    }

    /**
     * Creates a URL for cached DIMP data.
     *
     * @param string $type  The cache type.
     * @param string $cid   The cache id.
     *
     * @return string  The URL to the cache page.
     */
    function getCacheURL($type, $cid)
    {
        return Util::addParameter($GLOBALS['registry']->get('webroot', 'dimp') . '/cache.php', array('cid' => $cid, 'type' => $type));
    }

    /**
     * Outputs the necessary style tags, honoring local configuration
     * choices as to stylesheet caching.
     *
     * @private
     */
    function _includeStylesheetFiles()
    {
        $theme = $GLOBALS['prefs']->getValue('theme');
        $themesfs = $GLOBALS['registry']->get('themesfs', 'dimp');
        $themesuri = $GLOBALS['registry']->get('themesuri', 'dimp');
        $css = Horde::getStylesheets('dimp', $theme);
        $css_out = array();

        // Add print and browser-specific stylesheets.
        $css_out[] = array('u' => $themesuri . '/print.css',
                           'f' => $themesfs . '/print.css',
                           'm' => 'print');
        if (file_exists($themesfs . '/' . $theme . '/print.css')) {
            $css_out[] = array('u' => $themesuri . '/' . $theme . '/print.css',
                               'f' => $themesfs . '/' . $theme . '/print.css',
                               'm' => 'print');
        }

        // Load browser specific stylesheets if needed.
        if ($GLOBALS['browser']->isBrowser('msie')) {
            $ie_major = $GLOBALS['browser']->getMajor();
            $ie_css = null;
            if ($ie_major == 7) {
                $ie_css = '/ie7.css';
            } elseif ($ie_major < 7) {
                $ie_css = '/ie6_or_less.css';
            }

            if ($ie_css) {
                $css[] = array('u' => $themesuri . $ie_css,
                               'f' => $themesfs . $ie_css);
                if (file_exists($themesfs . '/' . $theme . $ie_css)) {
                    $css[] = array('u' => $themesuri . '/' . $theme . $ie_css,
                                   'f' => $themesfs . '/' . $theme . $ie_css);
                }
            }
        }

        // Load custom stylesheets.
        if (!empty($GLOBALS['conf']['css_files'])) {
            foreach ($GLOBALS['conf']['css_files'] as $css_file) {
                $css[] = array('u' => $themesuri . '/' . $css_file,
                               'f' => $themesfs .  '/' . $css_file);
            }
        }

        if (empty($GLOBALS['conf']['server']['cachecss']) ||
            empty($GLOBALS['conf']['cache']['driver'])) {
            $css_out = array_merge($css, $css_out);
        } else {
            require_once 'Horde/Cache.php';
            require_once DIMP_BASE . '/lib/version.php';
            $cache = &Horde_Cache::singleton($GLOBALS['conf']['cache']['driver'], Horde::getDriverConfig('cache', $GLOBALS['conf']['cache']['driver']));

            $mtime = 0;
            $out = '';

            if (!empty($GLOBALS['conf']['server']['cachecssparams']['mtime'])) {
                foreach ($css as $file) {
                    $mtime = max($mtime, filemtime($file['f']));
                }
            }

            $sig = md5(serialize($css) . $mtime . DIMP_VERSION);
            $exists = $cache->exists($sig, empty($GLOBALS['conf']['server']['cachecssparams']['lifetime']) ? 0 : $GLOBALS['conf']['server']['cachecssparams']['lifetime']);

            if (!$exists) {
                $flags = defined('FILE_IGNORE_NEW_LINES') ? (FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES) : 0;
                foreach ($css as $file) {
                    $path = substr($file['u'], 0, strrpos($file['u'], '/') + 1);
                    // Fix relative URLs, remove multiple whitespaces, and
                    // strip comments.
                    $out .= preg_replace(array('/(url\(["\']?)([^\/])/i', '/\s+/', '/\/\*.*?\*\//'), array('$1' . $path . '$2', ' ', ''), implode('', file($file['f'], $flags)));
                }
                $cache->set($sig, $out);
            }

            $css_out = array_merge(array(array('u' => DIMP::getCacheURL('css', $sig))), $css_out);
        }

        foreach ($css_out as $file) {
            echo '<link href="' . $file['u'] . '" rel="stylesheet" type="text/css"' . (isset($file['m']) ? ' media="' . $file['m'] . '"' : '') . ' />' . "\n";
        }
    }

    /**
     * Displays the javascript notification popups.
     *
     * @var string $document  Where (JS window) should we display the
     *                        notifications?
     * @var string $dimpcore  Where is the DimpCore JS library located
     *                        (relative to the current JS window)?
     *
     * @return string  The notification JS code.
     */
    function notify($document = 'document', $dimpcore = 'DimpCore')
    {
        $GLOBALS['notification']->notify(array('listeners' => 'status'));
        return DIMP::notifyJSCode($GLOBALS['dimp_listener']->getStack(true));
    }

    /**
     * Output the javascript code needed to display notification popups.
     *
     * @var array $msgs       A stack of notification data.
     * @var string $document  Where (JS window) should we display the
     *                        notifications?
     * @var string $dimpcore  Where is the DimpCore JS library located
     *                        (relative to the current JS window)?
     *
     * @return string  The notification JS code.
     */
    function notifyJSCode($msgs, $document = 'document',
                          $dimpcore = 'DimpCore')
    {
        if (!count($msgs)) {
            return '';
        }

        require_once 'Horde/Serialize.php';
        return $dimpcore . '.showNotifications(' .
               Horde_Serialize::serialize($msgs, SERIALIZE_JSON) .
               ', ' . $document . ');';
    }

    /**
     * Formats the response to send to javascript code when dealing with
     * folder operations.
     *
     * @param IMP_Tree $imptree  An IMP_Tree object.
     * @param array $changes     An array with three sub arrays - to be used
     *                           instead of the return from
     *                           $imptree->eltDiff():
     *                           'a' - a list of folders to add
     *                           'c' - a list of changed folders
     *                           'd' - a list of folders to delete
     *
     * @return array  The object used by the JS code to update the folder tree.
     */
    function getFolderResponse($imptree, $changes = null)
    {
        if ($changes === null) {
            $changes = $imptree->eltDiff();
        }
        if (empty($changes)) {
            return false;
        }

        $result = array('a' => array(), 'c' => array(), 'd' => array());

        foreach ($changes['a'] as $val) {
            $result['a'][] = DIMP::_createFolderElt($imptree->element($val));
        }

        foreach ($changes['c'] as $val) {
            // Skip the base element, since any change there won't ever be
            // updated on-screen.
            if ($val == IMPTREE_BASE_ELT) {
                continue;
            }
            $result['c'][] = DIMP::_createFolderElt($imptree->element($val));
        }

        foreach (array_reverse($changes['d']) as $val) {
            $result['d'][] = rawurlencode($val);
        }

        return $result;
    }

    /**
     * Create an object used by DimpCore to generate the folder tree.
     *
     * @access private
     *
     * @param array $elt  The output from IMP_Tree::element().
     *
     * @return stdClass  The element object. Contains the following items:
     * <pre>
     * 'ch' (children) = Does the folder contain children?
     * 'cl' (class) = The CSS class.
     * 'co' (container) = Is this folder a container element?
     * 'i' (icon) = A user defined icon to use.
     * 'l' (label) = The folder display label.
     * 'm' (mbox) = The mailbox value.
     * 'pa' (parent) = The parent element.
     * 'po' (polled) = Is the element polled?
     * 's' (special) = Is this a "special" element?
     * 'u' (unseen) = The number of unseen messages.
     * </pre>
     */
    function _createFolderElt($elt)
    {
        $ob = new stdClass;
        $ob->ch = $elt['children'];
        $ob->cl = 'base';
        $ob->co = $elt['container'];
        $ob->i = '';
        $ob->l = $elt['base_elt']['l'];
        $ob->m = rawurlencode($elt['value']);
        $ob->pa = rawurlencode($elt['parent']);
        $ob->po = $elt['polled'];
        $ob->s = false;

        if (!$ob->co) {
            if ($elt['polled']) {
                $ob->u = $elt['unseen'];
            }

            switch ($elt['special']) {
            case IMPTREE_SPECIAL_INBOX:
                $ob->cl = 'inbox';
                $ob->s = true;
                break;

            case IMPTREE_SPECIAL_TRASH:
                $ob->cl = 'trash';
                $ob->s = true;
                break;

            case IMPTREE_SPECIAL_SPAM:
                $ob->cl = 'spam';
                $ob->s = true;
                break;

            case IMPTREE_SPECIAL_DRAFT:
                $ob->cl = 'drafts';
                $ob->s = true;
                break;

            case IMPTREE_SPECIAL_SENT:
                $ob->cl = 'sent';
                $ob->s = true;
                break;

            default:
                if (!empty($elt['vfolder'])) {
                    // TODO: Add Virtual Folder menus.
                    if ($GLOBALS['imp_search']->isVTrashFolder($elt['value'])) {
                        $ob->cl = 'trash';
                    } elseif ($GLOBALS['imp_search']->isVINBOXFolder($elt['value'])) {
                        $ob->cl = 'inbox';
                    }
                } elseif ($elt['children']) {
                    $ob->cl = 'exp';
                }
                break;
            }
        } else {
            $ob->cl = 'exp';
        }

        if ($elt['user_icon']) {
            $ob->cl = 'custom';
            $ob->i = Horde::img($elt['icon'], $elt['alt'], '', $elt['icondir']);
        }

        return $ob;
    }

    /**
     * Returns a stdClass response object with added notification information.
     *
     * @param string $data   The 'response' data.
     * @param boolean $auto  If true, DimpCore will automatically display the
     *                       notification.  If false, the callback handler
     *                       is responsible for displaying the notification.
     */
    function getNotificationResponse($data = null, $auto = true)
    {
        $response = new stdClass();
        $response->response = $data;
        $GLOBALS['notification']->notify(array('listeners' => 'status'));
        $response->msgs = $GLOBALS['dimp_listener']->getStack();
        $response->msgs_auto = (bool)$auto;
        return $response;
    }

    /**
     * Outputs javascript code to close the requested view..
     *
     * @param string $code     Any additional code to run before closing the
     *                         window.
     * @param boolean $opener  Should notifications occur in opener window?
     * @param string $close    The close function we should use.
     */
    function closeViewJS($code = null, $opener = true, $close = 'close()')
    {
        return '<script type="text/javascript">' .
            (($code !== null) ? ($code . "\n") : '') .
            DIMP::notify(false, ($opener) ? 'parent.opener.document' : 'parent.document', ($opener) ? 'parent.opener.DimpCore' : 'parent.DimpCore') . "\n" .
            'parent.' . $close . ';</script>';
    }

    /**
     * Return information about the current attachments for a message
     *
     * @var object IMP_Compose $imp_compose  An IMP_Compose object.
     *
     * @return array  An array of arrays with the following keys:
     * <pre>
     * 'number' - The current attachment number
     * 'name' - The HTML encoded attachment name
     * 'type' - The MIME type of the attachment
     * 'size' - The size of the attachment in KB (string)
     * </pre>
     */
    function getAttachmentInfo($imp_compose)
    {
        $fwd_list = array();

        if ($imp_compose->numberOfAttachments()) {
            foreach ($imp_compose->getAttachments() as $file_num => $mime) {
                $fwd_list[] = array(
                    'number' => $file_num,
                    'name' => htmlspecialchars($mime->getName(true, true)),
                    'type' => $mime->getType(),
                    'size' => $mime->getSize()
                );
            }
        }

        return $fwd_list;
    }

    /**
     * Create a message list string.
     * Format: "range_start:range_end,uid,uid,range_start2:range_end2,..."
     *
     * @param array $ids  The array of IDs.
     *
     * @return string  The message list string.
     */
    function toRangeString($uids)
    {
        $first = $last = null;
        $out = array();

        sort($uids, SORT_NUMERIC);

        foreach ($uids as $val) {
            if ($first) {
                if ($last + 1 == $val) {
                    $last = $val;
                } else {
                    $out[] = $first . ($last == $first ? '' : (':' . $last));
                    $first = $last = $val;
                }
            } else {
                $first = $last = $val;
            }
        }

        $out[] = $first . ($last == $first ? '' : (':' . $last));

        return implode(',', $out);
    }

    /**
     * Parse a message list string generated by Array.toRangeString().
     * Format: "range_start:range_end,uid,uid,range_start2:range_end2,..."
     *
     * @param string $msgstr  The message list string.
     *
     * @return array  An array of IDs.
     */
    function parseRangeString($msgstr)
    {
        $msglist = array();
        $msgstr = trim($msgstr);

        if (!empty($msgstr)) {
            foreach (explode(',', $msgstr) as $val) {
                $range = explode(':', $val);
                if (count($range) == 1) {
                    $msglist[] = intval($val);
                } else {
                    $msglist = array_merge($msglist, range(intval($range[0]), intval($range[1])));
                }
            }
        }
        return $msglist;
    }

    /**
     * Return a list of DIMP specific menu items.
     *
     * @return array  The array of menu items.
     */
    function menuList()
    {
        if (isset($GLOBALS['dimp_conf']['menu']['apps']) &&
            is_array($GLOBALS['dimp_conf']['menu']['apps']) &&
            count($GLOBALS['dimp_conf']['menu']['apps'])) {
            return $GLOBALS['dimp_conf']['menu']['apps'];
        } else {
            return array();
        }
    }

    /**
     * Returns the appropriate link to call the message composition screen.
     *
     * @param mixed $args   List of arguments to pass to compose.php. If this
     *                      is passed in as a string, it will be parsed as a
     *                      toaddress?subject=foo&cc=ccaddress (mailto-style)
     *                      string.
     * @param array $extra  Hash of extra, non-standard arguments to pass to
     *                      compose.php.
     *
     * @return string  The link to the message composition screen.
     */
    function composeLink($args = array(), $extra = array())
    {
        // IE 6 & 7 handles window.open() URL param strings differently if
        // triggered via an href or an onclick.  Since we have no hint
        // at this point where this link will be used, we have to always
        // encode the params and explicitly call rawurlencode() in
        // compose.php.
        $args = IMP::composeLinkArgs($args, $extra);
        $encode_args = array();
        foreach ($args as $k => $v) {
            $encode_args[$k] = rawurlencode($v);
        }
        return 'javascript:void(window.open(\'' . Util::addParameter(Horde::applicationUrl('compose.php'), $encode_args, null, false) . '\', \'\', \'width=820,height=610,status=1,scrollbars=yes,resizable=yes\'));';
    }

}
