<?php
/**
 * Compose view logic.
 *
 * $Horde: dimp/lib/Views/Compose.php,v 1.20.2.1 2008/01/02 11:30:18 jan Exp $
 *
 * Copyright 2005-2008 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @package DIMP
 */

require_once 'Horde/MIME.php';
require_once 'Horde/Identity.php';
require_once 'Horde/Serialize.php';
require_once 'Horde/Text/Filter.php';

class DIMP_Views_Compose {

    function Compose($args)
    {
        /* Load Identity. */
        $identity = &Identity::singleton(array('imp', 'imp'));
        $selected_identity = $identity->getDefault();
        $sent_mail_folder = $identity->getValue('sent_mail_folder');
        if (!empty($sent_mail_folder)) {
            $sent_mail_folder = htmlspecialchars('"' . IMP::displayFolder($sent_mail_folder) . '"');
        }

        /* Get user identities. */
        $all_sigs = $identity->getAllSignatures();
        foreach ($all_sigs as $ident => $sig) {
            $identities[] = array(
                // 0 = Plain text signature
                $sig,
                // 1 = HTML signature
                str_replace(' target="_blank"', '', Text_Filter::filter($sig, 'text2html', array('parselevel' => TEXT_HTML_MICRO_LINKURL, 'class' => null, 'callback' => null))),
                // 2 = Signature location
                (bool)$identity->getValue('sig_first', $ident),
                // 3 = Sent mail folder name
                $identity->getValue('sent_mail_folder', $ident),
                // 4 = Save in sent mail folder by default?
                (bool)$identity->saveSentmail($ident),
                // 5 = Sent mail display name
                IMP::displayFolder($identity->getValue('sent_mail_folder', $ident)),
                // 6 = Bcc addresses to add
                MIME::addrArray2String($identity->getBccAddresses($ident))
            );
        }

        $abook_url = 'window.open(\'' . Horde::applicationUrl('contacts.php') . '\', \'contacts\', \'toolbar=no,location=no,status=no,scrollbars=yes,resizable=yes,width=550,height=300,left=100,top=100\'); return false;';

        Horde::addScriptFile('compose.js', 'dimp', true);

        $js_onload = array();
        $draft_index = $messageCache = null;
        if (!empty($args['messageCache'])) {
            require_once IMP_BASE . '/lib/Compose.php';
            $imp_compose = &IMP_Compose::singleton($args['messageCache']);
            $draft_index = intval($imp_compose->saveDraftIndex());
            $messageCache = $args['messageCache'];

            if ($imp_compose->numberOfAttachments()) {
                foreach ($imp_compose->getAttachments() as $num => $mime) {
                    $js_onload[] = 'DimpCompose.addAttach(' . $num . ', \'' . addslashes($mime->getName(true, true)) . '\', \'' . addslashes($mime->getType()) . '\', \'' . addslashes($mime->getSize()) . "')";
                }
            }
        }

        $js_code = array(
            'DimpCompose.auto_save_interval_val = ' . intval($GLOBALS['prefs']->getValue('auto_save_drafts')),
            'DimpCompose.identities = ' . Horde_Serialize::serialize($identities, SERIALIZE_JSON),
            'DimpCompose.popup_window = ' . intval($args['popup']),
            'DimpCompose.qreply = ' . intval($args['qreply']),
        );


        $compose_html = $rte = false;
        if ($GLOBALS['browser']->hasFeature('rte')) {
            $compose_html = $GLOBALS['prefs']->getValue('compose_html');
            $rte = true;
            require_once 'Horde/Editor.php';

            switch ($GLOBALS['dimp_prefs']->getValue('jseditor')) {
            case 'xinha':
                $xinha_hide = $GLOBALS['dimp_prefs']->getValue('xinha_hide_buttons');
                $editor = &Horde_Editor::singleton('xinha', array('id' => 'message', 'config' => array('statusBar' => false, 'height' => '@raw@Element.getHeight(\'messageParent\') - DimpCompose.mp_padding'), 'hidebuttons' => (empty($xinha_hide)) ? array() : explode(',', $xinha_hide), 'no_notify' => true, 'no_autoload' => true));
                break;

            case 'fckeditor':
                $editor = &Horde_Editor::singleton('fckeditor', array('id' => 'message', 'no_notify' => true));
                $fck_buttons = $GLOBALS['dimp_prefs']->getValue('fckeditor_buttons');
                if (!empty($fck_buttons)) {
                    $js_onload[] = 'oFCKeditor.Config.CustomConfigurationsPath = "' . DIMP::getCacheURL('fckeditor', null) . '"';
                    $js_onload[] = 'oFCKeditor.ToolbarSet = "DimpToolbar"';
                }
                break;
            }
        }

        if (!empty($js_onload)) {
            $js_code[] = 'document.observe("dom:loaded", function() { ' . implode(';', $js_onload) . ' })';
        }
        IMP::addInlineScript($js_code);

        $allow_folders = $GLOBALS['conf']['user']['allow_folders'];
        $allow_receipts = $GLOBALS['conf']['compose']['allow_receipts'];
        $d_read = $GLOBALS['prefs']->getValue('disposition_request_read');
        $save_attach = $GLOBALS['prefs']->getValue('save_attachments');
        $save_sent_mail_locked = $GLOBALS['prefs']->isLocked('save_sent_mail');

        // Buffer output so that we can return a string from this function
        ob_start();
        require DIMP_TEMPLATES . '/chunks/compose.php';
        $out = ob_get_contents();
        ob_clean();

        if ($rte) {
            $out .= $editor->getJS();
        }
        return $out;
    }

}
